############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# (c) 2016, 2017, 2018#
# Licence APL2.0
#
############################################################
# standard libraries
import logging
from threading import Lock

# external packages
import numpy as np

# noinspection PyProtectedMember
import erfa
from skyfield.api import Angle

# local import

__all__ = [
    'J2000ToJNow',
    'J2000ToAltAz',
    'JNowToJ2000',
]

log = logging.getLogger()

lock = Lock()


def JNowToJ2000(ra, dec, timeJD):
    """
    JNowToJ2000 uses the ERFA library which actually is included in the astropy package for
    doing the transform. If future releases of astropy this might be a separate package and
    will be imported and used adequately.

    :param ra:
    :param dec:
    :param timeJD:
    :return:
    """

    if not isinstance(ra, Angle):
        return Angle(hours=0), Angle(degrees=0)

    if not isinstance(dec, Angle):
        return Angle(hours=0), Angle(degrees=0)

    with lock:
        ra = ra.radians
        dec = dec.radians

        ra = erfa.anp(ra + erfa.eo06a(timeJD.tt, 0.0))

        raConv, decConv, _ = erfa.atic13(ra, dec, timeJD.ut1, 0.0)

        ra = Angle(radians=raConv, preference='hours')
        dec = Angle(radians=decConv, preference='degrees')
        return ra, dec


def J2000ToJNow(ra, dec, timeJD):
    """
    J2000ToJNow uses the ERFA library which actually is included in the astropy package for
    doing the transform. If future releases of astropy this might be a separate package and
    will be imported and used adequately.

    :param ra:
    :param dec:
    :param timeJD:
    :return:
    """

    if not isinstance(ra, Angle):
        return Angle(hours=0), Angle(degrees=0)

    if not isinstance(dec, Angle):
        return Angle(hours=0), Angle(degrees=0)

    with lock:
        ra = ra.radians
        dec = dec.radians

        raConv, decConv, eo = erfa.atci13(ra, dec, 0, 0, 0, 0, timeJD.ut1, 0)

        raConv = erfa.anp(raConv - eo)
        ra = Angle(radians=raConv, preference='hours')
        dec = Angle(radians=decConv, preference='degrees')
        return ra, dec


def J2000ToAltAz(ra, dec, timeJD, location):
    """
    J2000ToAltAz uses the ERFA library which actually is included in the astropy package for
    doing the transform. If future releases of astropy this might be a separate package and
    will be imported and used adequately.

    Please be aware, that all refraction corrections are made in the mount itself, so all
    parameters which are related to RC are set to zero.

    :param ra:
    :param dec:
    :param timeJD:
    :param location:
    :return:
    """

    if not isinstance(ra, Angle):
        return Angle(degrees=0), Angle(degrees=0)

    if not isinstance(dec, Angle):
        return Angle(degrees=0), Angle(degrees=0)

    with lock:
        ra = ra.radians
        dec = dec.radians
        lat = location.latitude.radians
        lon = location.longitude.radians
        elevation = location.elevation.m

        aob, zob, hob, dob, rob, eo = erfa.atco13(ra,
                                                  dec,
                                                  0.0,
                                                  0.0,
                                                  0.0,
                                                  0.0,
                                                  timeJD.ut1,
                                                  0.0,
                                                  0,
                                                  lon,
                                                  lat,
                                                  elevation,
                                                  0.0,
                                                  0.0,
                                                  0.0,
                                                  0.0,
                                                  0.0,
                                                  0.0)
        decConv = np.pi / 2 - zob

        ra = Angle(radians=aob, preference='degrees')
        dec = Angle(radians=decConv, preference='degrees')

        return ra, dec



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries

# external packages
from PyQt5.QtGui import QColor
from PyQt5.Qt3DExtras import QDiffuseSpecularMaterial, QMetalRoughMaterial
from PyQt5.Qt3DExtras import QPhongAlphaMaterial, QPhongMaterial

# local import
from gui.utilities.stylesQtCss import Styles


class Materials(Styles):

    __all__ = ['Materials',
               ]

    """
    class Materials defines all used materials for the loaded stl models or the meshed build
    programmatically inside the simulator
    """

    def __init__(self):
        super().__init__()

        self.aluminiumS = QMetalRoughMaterial()
        self.aluminiumS.setBaseColor(QColor(127, 127, 127))
        self.aluminiumS.setMetalness(0.7)
        self.aluminiumS.setRoughness(0.5)

        self.aluminiumGrey = QMetalRoughMaterial()
        self.aluminiumGrey.setBaseColor(QColor(164, 164, 192))
        self.aluminiumGrey.setMetalness(0.3)
        self.aluminiumGrey.setRoughness(0.5)

        self.aluminiumB = QDiffuseSpecularMaterial()
        self.aluminiumB.setAmbient(QColor(64, 64, 128))
        self.aluminiumB.setDiffuse(QColor(64, 64, 128))

        self.aluminiumR = QDiffuseSpecularMaterial()
        self.aluminiumR.setAmbient(QColor(192, 64, 64))
        self.aluminiumR.setDiffuse(QColor(192, 64, 64))

        self.environ1 = QMetalRoughMaterial()
        self.environ1.setBaseColor(QColor(64, 128, 32))
        self.environ1.setMetalness(0.8)
        self.environ1.setRoughness(1)

        self.aluminium = QDiffuseSpecularMaterial()
        self.aluminium.setAmbient(QColor(164, 164, 164))
        self.aluminium.setDiffuse(QColor(164, 164, 164))

        self.white = QDiffuseSpecularMaterial()
        self.white.setAmbient(QColor(228, 228, 228))
        self.white.setShininess(0.7)

        self.glass = QDiffuseSpecularMaterial()
        self.glass.setAlphaBlendingEnabled(True)
        self.glass.setAmbient(QColor(0, 0, 112))
        self.glass.setDiffuse(QColor(128, 128, 192, 220))

        self.stainless = QDiffuseSpecularMaterial()
        self.stainless.setAmbient(QColor(224, 223, 219))
        self.stainless.setDiffuse(QColor(145, 148, 152))
        self.stainless.setSpecular(QColor(255, 255, 230))
        self.stainless.setShininess(0.9)

        self.dome1 = QPhongMaterial()
        self.dome1.setAmbient(self.COLOR_ASTRO)
        self.dome1.setDiffuse(self.COLOR_ASTRO)
        self.dome1.setSpecular(self.COLOR_ASTRO)
        self.dome1.setShininess(0.5)

        self.dome2 = QPhongMaterial()
        self.dome2.setAmbient(QColor(24, 108, 124))
        self.dome2.setDiffuse(QColor(24, 108, 124))
        self.dome2.setSpecular(QColor(24, 108, 124))
        self.dome2.setShininess(0.7)

        self.dome3 = QPhongMaterial()
        self.dome3.setAmbient(QColor(16, 72, 96))
        self.dome3.setDiffuse(QColor(16, 72, 96))
        self.dome3.setSpecular(QColor(16, 72, 96))
        self.dome3.setShininess(0.3)

        self.dome1t = QPhongAlphaMaterial()
        self.dome1t.setAmbient(QColor(16, 72, 96))
        self.dome1t.setDiffuse(QColor(16, 72, 96))
        self.dome1t.setSpecular(QColor(16, 72, 96))
        self.dome1t.setShininess(0.5)
        self.dome1t.setAlpha(0.5)

        self.dome2t = QPhongAlphaMaterial()
        self.dome2t.setAmbient(QColor(12, 54, 72))
        self.dome2t.setDiffuse(QColor(12, 54, 72))
        self.dome2t.setSpecular(QColor(12, 54, 72))
        self.dome2t.setShininess(0.7)
        self.dome2t.setAlpha(0.5)

        self.dome3t = QPhongAlphaMaterial()
        self.dome3t.setAmbient(QColor(8, 36, 48))
        self.dome3t.setDiffuse(QColor(8, 36, 48))
        self.dome3t.setSpecular(QColor(8, 36, 48))
        self.dome3t.setShininess(0.3)
        self.dome3t.setAlpha(0.5)

        self.points = QPhongMaterial()
        self.points.setAmbient(QColor(16, 128, 16))
        self.points.setDiffuse(QColor(16, 128, 16))
        self.points.setSpecular(QColor(16, 128, 16))
        self.points.setShininess(1)

        self.lines = QPhongMaterial()
        self.lines.setAmbient(QColor(96, 96, 0))
        self.lines.setDiffuse(QColor(96, 96, 0))
        self.lines.setSpecular(QColor(96, 96, 0))
        self.lines.setShininess(1.0)

        self.numbers = QPhongMaterial()
        self.numbers.setAmbient(QColor(16, 192, 16))
        self.numbers.setDiffuse(QColor(16, 192, 16))
        self.numbers.setSpecular(QColor(16, 192, 16))
        self.numbers.setShininess(1)

        self.pointer = QPhongMaterial()
        self.pointer.setAmbient(self.COLOR_PINK)
        self.pointer.setDiffuse(self.COLOR_PINK)
        self.pointer.setSpecular(self.COLOR_PINK)
        self.pointer.setShininess(1)

        self.walls = QPhongAlphaMaterial()
        self.walls.setAmbient(QColor(32, 128, 32))
        self.walls.setDiffuse(QColor(32, 128, 32))
        self.walls.setSpecular(QColor(32, 128, 32))
        self.walls.setShininess(0.8)
        self.walls.setAlpha(0.3)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/simulator.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SimulatorDialog(object):
    def setupUi(self, SimulatorDialog):
        SimulatorDialog.setObjectName("SimulatorDialog")
        SimulatorDialog.resize(800, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SimulatorDialog.sizePolicy().hasHeightForWidth())
        SimulatorDialog.setSizePolicy(sizePolicy)
        SimulatorDialog.setMinimumSize(QtCore.QSize(800, 600))
        SimulatorDialog.setMaximumSize(QtCore.QSize(1600, 1200))
        SimulatorDialog.setSizeIncrement(QtCore.QSize(10, 10))
        SimulatorDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        SimulatorDialog.setFont(font)
        self.horizontalLayout = QtWidgets.QHBoxLayout(SimulatorDialog)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout.setContentsMargins(0, -1, -1, -1)
        self.gridLayout.setObjectName("gridLayout")
        self.simulator = QtWidgets.QHBoxLayout()
        self.simulator.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.simulator.setSpacing(0)
        self.simulator.setObjectName("simulator")
        self.gridLayout.addLayout(self.simulator, 0, 1, 2, 1)
        self.groupBox = QtWidgets.QGroupBox(SimulatorDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setProperty("large", True)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setContentsMargins(8, 10, 16, 4)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.topView = QtWidgets.QPushButton(self.groupBox)
        self.topView.setMinimumSize(QtCore.QSize(0, 25))
        self.topView.setObjectName("topView")
        self.verticalLayout.addWidget(self.topView)
        self.topWestView = QtWidgets.QPushButton(self.groupBox)
        self.topWestView.setMinimumSize(QtCore.QSize(0, 25))
        self.topWestView.setObjectName("topWestView")
        self.verticalLayout.addWidget(self.topWestView)
        self.westView = QtWidgets.QPushButton(self.groupBox)
        self.westView.setMinimumSize(QtCore.QSize(0, 25))
        self.westView.setObjectName("westView")
        self.verticalLayout.addWidget(self.westView)
        self.topEastView = QtWidgets.QPushButton(self.groupBox)
        self.topEastView.setMinimumSize(QtCore.QSize(0, 25))
        self.topEastView.setObjectName("topEastView")
        self.verticalLayout.addWidget(self.topEastView)
        self.eastView = QtWidgets.QPushButton(self.groupBox)
        self.eastView.setMinimumSize(QtCore.QSize(0, 25))
        self.eastView.setObjectName("eastView")
        self.verticalLayout.addWidget(self.eastView)
        self.telescopeView = QtWidgets.QPushButton(self.groupBox)
        self.telescopeView.setMinimumSize(QtCore.QSize(0, 25))
        self.telescopeView.setObjectName("telescopeView")
        self.verticalLayout.addWidget(self.telescopeView)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(SimulatorDialog)
        self.groupBox_2.setProperty("large", True)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_2.setContentsMargins(8, 10, 16, 4)
        self.verticalLayout_2.setSpacing(4)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.checkDomeTransparent = QtWidgets.QCheckBox(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkDomeTransparent.sizePolicy().hasHeightForWidth())
        self.checkDomeTransparent.setSizePolicy(sizePolicy)
        self.checkDomeTransparent.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkDomeTransparent.setFont(font)
        self.checkDomeTransparent.setObjectName("checkDomeTransparent")
        self.verticalLayout_2.addWidget(self.checkDomeTransparent)
        self.checkShowPointer = QtWidgets.QCheckBox(self.groupBox_2)
        self.checkShowPointer.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowPointer.setFont(font)
        self.checkShowPointer.setObjectName("checkShowPointer")
        self.verticalLayout_2.addWidget(self.checkShowPointer)
        self.checkShowBuildPoints = QtWidgets.QCheckBox(self.groupBox_2)
        self.checkShowBuildPoints.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowBuildPoints.setFont(font)
        self.checkShowBuildPoints.setObjectName("checkShowBuildPoints")
        self.verticalLayout_2.addWidget(self.checkShowBuildPoints)
        self.checkShowNumbers = QtWidgets.QCheckBox(self.groupBox_2)
        self.checkShowNumbers.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowNumbers.setFont(font)
        self.checkShowNumbers.setObjectName("checkShowNumbers")
        self.verticalLayout_2.addWidget(self.checkShowNumbers)
        self.checkShowSlewPath = QtWidgets.QCheckBox(self.groupBox_2)
        self.checkShowSlewPath.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowSlewPath.setFont(font)
        self.checkShowSlewPath.setObjectName("checkShowSlewPath")
        self.verticalLayout_2.addWidget(self.checkShowSlewPath)
        self.checkShowHorizon = QtWidgets.QCheckBox(self.groupBox_2)
        self.checkShowHorizon.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowHorizon.setFont(font)
        self.checkShowHorizon.setObjectName("checkShowHorizon")
        self.verticalLayout_2.addWidget(self.checkShowHorizon)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.gridLayout.addWidget(self.groupBox_2, 1, 0, 1, 1)
        self.gridLayout.setColumnStretch(1, 1)
        self.horizontalLayout.addLayout(self.gridLayout)

        self.retranslateUi(SimulatorDialog)
        QtCore.QMetaObject.connectSlotsByName(SimulatorDialog)

    def retranslateUi(self, SimulatorDialog):
        _translate = QtCore.QCoreApplication.translate
        SimulatorDialog.setWindowTitle(_translate("SimulatorDialog", "Mount Simulation View"))
        self.groupBox.setToolTip(_translate("SimulatorDialog", "<html><head/><body><p><span style=\" font-weight:600;\">Left mouse button :</span></p><p>While the left mouse button is pressed, mouse movement along x-axis moves the camera left and right and movement along y-axis moves it up and down.</p><p><span style=\" font-weight:600;\">Right mouse button :</span></p><p>While the right mouse button is pressed, mouse movement along x-axis pans the camera around the camera view center and movement along y-axis tilts it around the camera view center.</p><p><span style=\" font-weight:600;\">Both left and right mouse button </span>:</p><p>While both the left and the right mouse button are pressed, mouse movement along y-axis zooms the camera in and out without changing the view center.</p><p><span style=\" font-weight:600;\">Mouse scroll wheel :</span></p><p>Zooms the camera in and out without changing the view center.</p><p><span style=\" font-weight:600;\">Arrow keys :</span></p><p>Move the camera vertically and horizontally relative to camera viewport.</p><p><span style=\" font-weight:600;\">Page up and page down keys :</span></p><p>Move the camera forwards and backwards.</p><p><span style=\" font-weight:600;\">Shift key : </span></p><p>Changes the behavior of the up and down arrow keys to zoom the camera in and out without changing the view center. The other movement keys are disabled.</p><p><span style=\" font-weight:600;\">Alt key :</span></p><p>Changes the behovior of the arrow keys to pan and tilt the camera around the view center. Disables the page up and page down keys.</p><p><span style=\" font-weight:600;\">Escape :</span></p><p>Moves the camera so that entire scene is visible in the camera viewport.</p></body></html>"))
        self.groupBox.setTitle(_translate("SimulatorDialog", "Select View"))
        self.topView.setText(_translate("SimulatorDialog", "Top View"))
        self.topWestView.setText(_translate("SimulatorDialog", "Top West View"))
        self.westView.setText(_translate("SimulatorDialog", "West View"))
        self.topEastView.setText(_translate("SimulatorDialog", "Top East View"))
        self.eastView.setText(_translate("SimulatorDialog", "East View"))
        self.telescopeView.setText(_translate("SimulatorDialog", "Telescope View"))
        self.groupBox_2.setTitle(_translate("SimulatorDialog", "Show Settings"))
        self.checkDomeTransparent.setText(_translate("SimulatorDialog", "Dome transp."))
        self.checkShowPointer.setText(_translate("SimulatorDialog", "Intersect Point"))
        self.checkShowBuildPoints.setText(_translate("SimulatorDialog", "Build Points"))
        self.checkShowNumbers.setText(_translate("SimulatorDialog", "Numbers"))
        self.checkShowSlewPath.setText(_translate("SimulatorDialog", "Slew Path"))
        self.checkShowHorizon.setText(_translate("SimulatorDialog", "Horizon"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    SimulatorDialog = QtWidgets.QWidget()
    ui = Ui_SimulatorDialog()
    ui.setupUi(SimulatorDialog)
    SimulatorDialog.show()
    sys.exit(app.exec_())



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries
import threading
from dateutil.tz import tzlocal

# external packages
from PyQt5.QtGui import QPixmap, QPainter, QPen, QColor, QFont
from PyQt5.QtCore import Qt, QPointF
from skyfield import almanac
import numpy as np

# local import


class Almanac:
    """
    """

    phasesText = {
        'New moon': {
            'range': (0, 1),
        },
        'Waxing crescent': {
            'range': (1, 23),
        },
        'First Quarter': {
            'range': (23, 27),
        },
        'Waxing Gibbous': {
            'range': (27, 48),
        },
        'Full moon': {
            'range': (48, 52),
        },
        'Waning Gibbous': {
            'range': (52, 73),
        },
        'Third quarter': {
            'range': (73, 77),
        },
        'Waning crescent': {
            'range': (77, 99),
        },
        'New moon ': {
            'range': (99, 100),
        },
    }

    def __init__(self):

        self.civil = None
        self.nautical = None
        self.astronomical = None
        self.dark = None
        self.thread = None

        self.app.mount.signals.locationDone.connect(self.searchTwilight)
        self.app.mount.signals.locationDone.connect(self.displayTwilightData)
        self.app.update30m.connect(self.updateMoonPhase)

    def initConfig(self):
        """
        initConfig read the key out of the configuration dict and stores it to the gui
        elements. if some initialisations have to be proceeded with the loaded persistent
        data, they will be launched as well in this method.

        :return: True for test purpose
        """

        self.updateMoonPhase()
        self.searchTwilight()
        self.displayTwilightData()
        self.lunarNodes()

        return True

    def storeConfig(self):
        """
        storeConfig writes the keys to the configuration dict and stores. if some
        saving has to be proceeded to persistent data, they will be launched as
        well in this method.

        :return: True for test purpose
        """
        if self.thread:
            self.thread.join()

        return True

    def drawTwilight(self, minDay, maxDay):
        """
        draw Twilight takes the different stats for each day during a year (half a year to the past,
        half year to the future) and draws the chart for UTC time only.

        :return: true for test purpose
        """

        if minDay is None or maxDay is None:
            return False

        ts = self.app.mount.obsSite.ts
        minLim = int(minDay.tt) + 1
        maxLim = int(maxDay.tt) - 1
        midLim = minLim + (maxLim - minLim) / 2

        axe, fig = self.generateFlat(widget=self.twilight)

        yTicks = [0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24]
        yLabels = ['12', '14', '16', '18', '20', '22', '24',
                   '02', '04', '06', '08', '10', '12']

        axe.set_yticks(yTicks)
        axe.set_yticklabels(yLabels, fontsize=10)

        xTicks = np.arange(minLim, maxLim, (maxLim - minLim) / 11)
        xLabels = ts.tt_jd(xTicks).utc_strftime('%d%b')
        xLabels[0] = ''
        axe.set_xlim(minLim, maxLim)

        axe.set_xticks(xTicks)
        axe.set_xticklabels(xLabels, fontsize=10)

        axe.grid(color=self.M_GREY, alpha=0.5)
        axe.set_ylim(0, 24)

        for i in self.civil:
            val = self.civil[i]
            if not len(val):
                break
            val.append(val[0])
            x = [x[0].tt for x in val]
            y = [x[1] for x in val]
            axe.fill(x, y, self.M_BLUE1)
            axe.plot(x, y, self.M_BLUE1)

        for i in self.nautical:
            val = self.nautical[i]
            if not len(val):
                break
            val.append(val[0])
            x = [x[0].tt for x in val]
            y = [x[1] for x in val]
            axe.fill(x, y, self.M_BLUE2)
            axe.plot(x, y, self.M_GREY)

        for i in self.astronomical:
            val = self.astronomical[i]
            if not len(val):
                break
            val.append(val[0])
            x = [x[0].tt for x in val]
            y = [x[1] for x in val]
            axe.fill(x, y, self.M_BLUE3)
            axe.plot(x, y, self.M_GREY)

        for i in self.dark:
            val = self.dark[i]
            if not len(val):
                break
            val.append(val[0])
            x = [x[0].tt for x in val]
            y = [x[1] for x in val]
            axe.fill(x, y, self.M_BLUE4)
            axe.plot(x, y, self.M_GREY)

        x = [midLim - 2, midLim - 2, midLim + 2, midLim + 2]
        y = [0, 24, 24, 0]
        axe.fill(x, y, self.M_GREY, alpha=0.5)

        # draw legend
        d = 0
        x = [minLim + 5 + d, minLim + 5 + d, minLim + 85 + d,
             minLim + 85 + d, minLim + 5 + d]
        y = [22.5, 23.5, 23.5, 22.5, 22.5]
        axe.fill(x, y, self.M_BLUE1)
        axe.plot(x, y, self.M_GREY)
        axe.annotate('Civil', (minLim + 12 + d, 22.75),
                     color=self.M_WHITE, weight='bold')

        d = 90
        x = [minLim + 5 + d, minLim + 5 + d, minLim + 85 + d,
             minLim + 85 + d, minLim + 5 + d]
        axe.fill(x, y, self.M_BLUE2)
        axe.plot(x, y, self.M_GREY)
        axe.annotate('Nautical', (minLim + 12 + d, 22.75),
                     color=self.M_WHITE, weight='bold')

        d = 180
        x = [minLim + 5 + d, minLim + 5 + d, minLim + 85 + d,
             minLim + 85 + d, minLim + 5 + d]
        axe.fill(x, y, self.M_BLUE3)
        axe.plot(x, y, self.M_GREY)
        axe.annotate('Astronomical', (minLim + 12 + d, 22.75),
                     color=self.M_WHITE, weight='bold')

        d = 270
        x = [minLim + 5 + d, minLim + 5 + d, minLim + 85 + d,
             minLim + 85 + d, minLim + 5 + d]
        axe.fill(x, y, self.M_BLUE4)
        axe.plot(x, y, self.M_GREY)
        axe.annotate('Dark', (minLim + 12 + d, 22.75),
                     color=self.M_WHITE, weight='bold')

        x = [midLim - 20, midLim - 20, midLim + 20, midLim + 20, midLim - 20]
        y = [0.5, 1.5, 1.5, 0.5, 0.5]
        axe.fill(x, y, self.M_GREY, alpha=0.5)
        axe.plot(x, y, self.M_GREY)
        axe.annotate('Today', (midLim - 12, 0.75),
                     color=self.M_WHITE, weight='bold')

        axe.figure.canvas.draw()

        return True

    def calcTwilightData(self, timeWindow=0):
        """
        :param timeWindow:
        :return:
        """
        ts = self.app.mount.obsSite.ts
        timeJD = self.app.mount.obsSite.timeJD
        location = self.app.mount.obsSite.location

        if location is None:
            return [], [], None

        t0 = ts.tt_jd(int(timeJD.tt) - timeWindow)
        t1 = ts.tt_jd(int(timeJD.tt) + timeWindow + 1)

        f = almanac.dark_twilight_day(self.app.ephemeris, location)
        timeEvents, events = almanac.find_discrete(t0, t1, f)

        return timeEvents, events, t0

    def searchTwilightWorker(self, timeWindow):
        """
        searchTwilightWorker is the worker method which does the search for twilight events
        during one year with actual day as middle point. As this search take some time and
        the gui should be still responsive, this method will run in a separate thread.

        :return: true for test purpose
        """
        t, e, t0 = self.calcTwilightData(timeWindow)
        if t0 is None:
            return False

        self.civil = {0: []}
        civilP = 0
        self.nautical = {0: []}
        nauticalP = 0
        self.astronomical = {0: []}
        astronomicalP = 0
        self.dark = {0: []}
        darkP = 0
        civilE = False
        nauticalE = False
        astronomicalE = False
        darkE = False
        stat = 4
        lastDay = round(t0.tt + 0.5, 0)
        minDay = None

        for ti, event in zip(t, e):
            hour = int(ti.astimezone(tzlocal()).strftime('%H'))
            minute = int(ti.astimezone(tzlocal()).strftime('%M'))

            y = (hour + 12 + minute / 60) % 24
            day = round(ti.tt + 0.5, 0)

            if minDay is None:
                minDay = ti

            if day != lastDay:
                if not civilE:
                    if len(self.civil[civilP]):
                        civilP += 1
                        self.civil[civilP] = []
                if not nauticalE:
                    if len(self.nautical[nauticalP]):
                        nauticalP += 1
                        self.nautical[nauticalP] = []
                if not astronomicalE:
                    if len(self.astronomical[astronomicalP]):
                        astronomicalP += 1
                        self.astronomical[astronomicalP] = []
                if not darkE:
                    if len(self.dark[darkP]):
                        darkP += 1
                        self.dark[darkP] = []
                lastDay = day
                civilE = False
                nauticalE = False
                astronomicalE = False
                darkE = False

            if stat == 4 and event == 3:
                self.civil[civilP].append([ti, y])
                civilE = True
            elif stat == 3 and event == 2:
                self.nautical[nauticalP].append([ti, y])
                nauticalE = True
            elif stat == 2 and event == 1:
                self.astronomical[astronomicalP].append([ti, y])
                astronomicalE = True
            elif stat == 1 and event == 0:
                self.dark[darkP].append([ti, y])
                darkE = True
            elif stat == 0 and event == 1:
                self.dark[darkP].insert(0, [ti, y])
                darkE = True
            elif stat == 1 and event == 2:
                self.astronomical[astronomicalP].insert(0, [ti, y])
                astronomicalE = True
            elif stat == 2 and event == 3:
                self.nautical[nauticalP].insert(0, [ti, y])
                nauticalE = True
            elif stat == 3 and event == 4:
                self.civil[civilP].insert(0, [ti, y])
                civilE = True

            stat = event
            maxDay = ti

        self.drawTwilight(minDay, maxDay)

        return True

    def searchTwilight(self):
        """
        search twilight just starts the worker in a separate thread.

        :return: true for test purpose
        """

        self.thread = threading.Thread(target=self.searchTwilightWorker, args=[182])
        self.thread.start()

        return True

    def displayTwilightData(self):
        """
        displayTwilightData populates the readable list of twilight events in the upcoming
        observation night. the user could choose the timezone local or UTC

        :return: true for test purpose
        """

        colors = {
            0: self.COLOR_BLUE4,
            1: self.COLOR_BLUE3,
            2: self.COLOR_BLUE2,
            3: self.COLOR_BLUE1,
            4: self.COLOR_WHITE1,
        }

        timeEvents, events, _ = self.calcTwilightData()

        text = ''
        self.ui.twilightEvents.clear()

        for timeEvent, event in zip(timeEvents, events):
            self.ui.twilightEvents.setTextColor(colors[event])
            self.ui.twilightEvents.setFontWeight(QFont.Bold)
            text += f'{timeEvent.astimezone(tzlocal()).strftime("%H:%M:%S")} '
            text += f'{almanac.TWILIGHTS[event]}'
            self.ui.twilightEvents.insertPlainText(text)
            text = '\n'

        return True

    def getMoonPhase(self):
        """
        updateMoonPhase searches for moon events, moon phase and illumination percentage.

        :return: fraction of illumination, degree phase and percent phase
        """

        sun = self.app.ephemeris['sun']
        moon = self.app.ephemeris['moon']
        earth = self.app.ephemeris['earth']
        now = self.app.mount.obsSite.ts.now()

        e = earth.at(self.app.mount.obsSite.timeJD)
        _, sunLon, _ = e.observe(sun).apparent().ecliptic_latlon()
        _, moonLon, _ = e.observe(moon).apparent().ecliptic_latlon()

        mpIllumination = almanac.fraction_illuminated(self.app.ephemeris, 'moon', now)
        mpDegree = (moonLon.degrees - sunLon.degrees) % 360.0
        mpPercent = mpDegree / 360

        return mpIllumination, mpDegree, mpPercent

    def updateMoonPhase(self):
        """
        It will also write some description for the moon phase. In addition I will show an
        image of the moon showing the moon phase as picture.

        :return: true for test purpose
        """

        mpIllumination, mpDegree, mpPercent = self.getMoonPhase()

        self.ui.moonPhaseIllumination.setText(f'{mpIllumination * 100:3.2f}')
        self.ui.moonPhasePercent.setText(f'{100* mpPercent:3.0f}')
        self.ui.moonPhaseDegree.setText(f'{mpDegree:3.0f}')

        for phase in self.phasesText:
            if int(mpPercent * 100) not in range(*self.phasesText[phase]['range']):
                continue
            self.ui.moonPhaseText.setText(phase)

        width = self.ui.moonPic.width()
        height = self.ui.moonPic.height()

        mask = QPixmap(width, height)
        maskP = QPainter(mask)
        maskP.setBrush(Qt.SolidPattern)
        maskP.setBrush(QColor(255, 255, 255))
        maskP.setPen(QPen(QColor(255, 255, 255)))
        maskP.drawRect(0, 0, width, height)

        if 0 <= mpDegree <= 90:
            maskP.setBrush(QColor(48, 48, 48))
            maskP.drawPie(0, 0, width, height, 90 * 16, 180 * 16)

            r = np.cos(np.radians(mpDegree)) * width / 2
            maskP.setBrush(QColor(48, 48, 48))
            maskP.setPen(QPen(QColor(48, 48, 48)))
            maskP.drawEllipse(QPointF(width / 2, height / 2), r, height / 2)

        elif 90 < mpDegree <= 180:
            maskP.setBrush(QColor(48, 48, 48))
            maskP.drawPie(0, 0, width, height, 90 * 16, 180 * 16)

            r = - np.cos(np.radians(mpDegree)) * width / 2
            maskP.setBrush(QColor(255, 255, 255))
            maskP.setPen(QPen(QColor(255, 255, 255)))
            maskP.drawEllipse(QPointF(width / 2, height / 2), r, height / 2)

        elif 180 < mpDegree <= 270:
            maskP.setBrush(QColor(48, 48, 48))
            maskP.drawPie(0, 0, width, height, - 90 * 16, 180 * 16)

            r = - np.cos(np.radians(mpDegree)) * width / 2
            maskP.setBrush(QColor(255, 255, 255))
            maskP.setPen(QPen(QColor(255, 255, 255)))
            maskP.drawEllipse(QPointF(width / 2, height / 2), r, height / 2)

        else:
            maskP.setBrush(QColor(48, 48, 48))
            maskP.drawPie(0, 0, width, height, -90 * 16, 180 * 16)

            r = np.cos(np.radians(mpDegree)) * width / 2
            maskP.setPen(QPen(QColor(48, 48, 48)))
            maskP.setBrush(QColor(48, 48, 48))
            maskP.drawEllipse(QPointF(width / 2, height / 2), r, height / 2)

        maskP.end()

        moon = QPixmap(':/pics/moon.png').scaled(width, height)
        moonP = QPainter(moon)
        moonP.setCompositionMode(QPainter.CompositionMode_Multiply)
        moonP.drawPixmap(0, 0, mask)
        moonP.end()

        self.ui.moonPic.setPixmap(moon)

        return True

    def lunarNodes(self):
        """

        :return: true for test purpose
        """
        ts = self.app.mount.obsSite.ts
        timeJD = self.app.mount.obsSite.timeJD

        t0 = ts.tt_jd(int(timeJD.tt))
        t1 = ts.tt_jd(int(timeJD.tt) + 29)
        t, y = almanac.find_discrete(t0, t1, almanac.moon_nodes(self.app.ephemeris))

        text = 'descending' if y[0] else 'ascending'
        self.ui.lunarNodes.setText(f'{text}')

        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries
import time

# external packages
from PyQt5.QtGui import QTextCursor
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.uic

# local import
from gui.utilities import toolsQtWidget
from gui.widgets import message_ui


class MessageWindow(toolsQtWidget.MWidget):
    """
    the message window class handles

    """

    __all__ = ['MessageWindow',
               ]

    def __init__(self, app):
        super().__init__()

        self.app = app
        self.ui = message_ui.Ui_MessageDialog()
        self.ui.setupUi(self)
        self.initUI()

        self.messColor = [self.COLOR_ASTRO,
                          self.COLOR_WHITE,
                          self.COLOR_YELLOW,
                          self.COLOR_RED,
                          ]
        self.messFont = [PyQt5.QtGui.QFont.Normal,
                         PyQt5.QtGui.QFont.Bold,
                         PyQt5.QtGui.QFont.Normal,
                         PyQt5.QtGui.QFont.Normal,
                         ]

    def initConfig(self):
        """
        initConfig read the key out of the configuration dict and stores it to the gui
        elements. if some initialisations have to be proceeded with the loaded persistent
        data, they will be launched as well in this method.

        :return: True for test purpose
        """

        if 'messageW' not in self.app.config:
            self.app.config['messageW'] = {}
        config = self.app.config['messageW']
        x = config.get('winPosX', 140)
        y = config.get('winPosY', 140)
        if x > self.screenSizeX:
            x = 0
        if y > self.screenSizeY:
            y = 0
        self.move(x, y)
        height = config.get('height', 600)
        self.resize(800, height)

        return True

    def storeConfig(self):
        """
        storeConfig writes the keys to the configuration dict and stores. if some
        saving has to be proceeded to persistent data, they will be launched as
        well in this method.

        :return: True for test purpose
        """
        if 'messageW' not in self.app.config:
            self.app.config['messageW'] = {}
        config = self.app.config['messageW']
        config['winPosX'] = self.pos().x()
        config['winPosY'] = self.pos().y()
        config['height'] = self.height()

        return True

    def closeEvent(self, closeEvent):
        self.storeConfig()

        # gui signals
        self.ui.clear.clicked.disconnect(self.clearWindow)
        self.app.update1s.disconnect(self.writeMessage)

        super().closeEvent(closeEvent)

    def showWindow(self):
        self.show()

        # gui signals
        self.ui.clear.clicked.connect(self.clearWindow)
        self.app.update1s.connect(self.writeMessage)

    def clearWindow(self):
        """
        clearWindow resets the window and shows empty text.

        :return: true for test purpose
        """

        self.ui.message.clear()
        return True

    def writeMessage(self):
        """
        writeMessage takes singles with message and writes them to the text browser window.
        types:
            0: normal text
            1: highlighted text
            2: warning text
            3: error text

        :return: true for test purpose
        """

        while not self.app.messageQueue.empty():
            message, mType = self.app.messageQueue.get()

            if mType < 0:
                continue

            if mType > len(self.messColor):
                continue

            prefix = time.strftime('%H:%M:%S ', time.localtime())

            self.ui.message.setTextColor(self.messColor[mType])
            self.ui.message.setFontWeight(self.messFont[mType])
            self.ui.message.insertPlainText(prefix + message + '\n')
            self.ui.message.moveCursor(QTextCursor.End)

        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries

# external packages

# local imports
from base.alpacaClass import AlpacaClass
from base.alpacaBase import FilterWheel


class FilterAlpaca(AlpacaClass):
    """
    the class filter inherits all information and handling of the filter device.
    """

    __all__ = ['FilterAlpaca',
               ]

    # specific timing for device
    CYCLE_DEVICE = 3000
    CYCLE_DATA = 1000

    def __init__(self, app=None, signals=None, data=None):
        super().__init__(app=app, data=data, threadPool=app.threadPool)

        # as we have in the base class only the base client there, we will get more
        # specialized with Dome (which is derived from the base class)
        self.client = FilterWheel()
        self.signals = signals
        self.data = data

    def getInitialConfig(self):
        """
        :return: success
        """
        super().getInitialConfig()
        names = self.client.names()
        if names is None:
            return False

        for i, name in enumerate(names):
            if name is None:
                continue
            self.data[f'FILTER_NAME.FILTER_SLOT_NAME_{i:1.0f}'] = name

        return True

    def workerPollData(self):
        """
        :return: true for test purpose
        """
        if not self.deviceConnected:
            return False

        position = self.client.position()
        if position == -1 or position is None:
            return False

        self.data['FILTER_SLOT.FILTER_SLOT_VALUE'] = position
        return True

    def sendFilterNumber(self, filterNumber=0):
        """
        :return: true for test purpose
        """
        if not self.deviceConnected:
            return False

        self.client.position(Position=filterNumber)
        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import logging

# external packages
import PyQt5
import numpy as np

# local imports
from logic.measure.measureRaw import MeasureDataRaw
from logic.measure.measureCSV import MeasureDataCSV


class MeasureSignals(PyQt5.QtCore.QObject):
    """
    The DomeSignals class offers a list of signals to be used and instantiated by
    the Mount class to get signals for triggers for finished tasks to
    enable a gui to update their values transferred to the caller back.

    This has to be done in a separate class as the signals have to be subclassed from
    QObject and the Mount class itself is subclassed from object
    """

    __all__ = ['MeasureSignals']

    serverConnected = PyQt5.QtCore.pyqtSignal()
    serverDisconnected = PyQt5.QtCore.pyqtSignal(object)
    deviceConnected = PyQt5.QtCore.pyqtSignal(str)
    deviceDisconnected = PyQt5.QtCore.pyqtSignal(str)


class MeasureData(PyQt5.QtCore.QObject):
    """
    the class MeasureData inherits all information and handling of data management and
    storage

        >>> measure = MeasureData(
        >>>                 )
    """

    __all__ = ['MeasureData',
               ]

    log = logging.getLogger(__name__)

    # update rate to 1 seconds for setting indi server
    CYCLE_UPDATE_TASK = 1000
    # maximum size of measurement task
    MAXSIZE = 24 * 60 * 60

    def __init__(self, app):
        super().__init__()
        self.app = app
        self.signals = MeasureSignals()
        self.mutexMeasure = PyQt5.QtCore.QMutex()

        # internal calculations
        self.shorteningStart = True
        self.raRef = None
        self.decRef = None
        self.angularPosRaRef = None
        self.angularPosDecRef = None
        self.devices = {}
        self.deviceName = ''

        self.data = {}
        self.defaultConfig = {'framework': '',
                              'frameworks': {}}
        self.run = {
            'raw': MeasureDataRaw(self.app, self, self.data),
            'csv': MeasureDataCSV(self.app, self, self.data)
        }
        for fw in self.run:
            self.defaultConfig['frameworks'].update(self.run[fw].defaultConfig)
        self.framework = ''

    def startCommunication(self, loadConfig=False):
        """
        :param loadConfig:
        :return: True for test purpose
        """
        deviceStat = self.app.deviceStat
        dItems = deviceStat.items()
        self.devices = [key for key, value in dItems if deviceStat[key] is not None]

        if self.framework not in self.run.keys():
            return False

        name = self.run[self.framework].deviceName
        suc = self.run[self.framework].startCommunication(loadConfig=loadConfig)
        if suc:
            self.signals.deviceConnected.emit(name)
            self.app.message.emit(f'MEASURE found:       [{name}]', 0)

        return suc

    def stopCommunication(self):
        """
        :return: true for test purpose
        """
        if self.framework not in self.run.keys():
            return False

        suc = self.run[self.framework].stopCommunication()
        name = self.run[self.framework].deviceName
        self.signals.serverDisconnected.emit({name: 0})
        self.signals.deviceDisconnected.emit(name)
        self.app.message.emit(f'MEASURE remove:      [{name}]', 0)
        return suc

    def setEmptyData(self):
        """
        :return: True for test purpose
        """
        self.data.clear()
        self.data['time'] = np.empty(shape=[0, 1], dtype='datetime64')
        self.data['deltaRaJNow'] = np.empty(shape=[0, 1])
        self.data['deltaDecJNow'] = np.empty(shape=[0, 1])
        self.data['deltaAngularPosRA'] = np.empty(shape=[0, 1])
        self.data['deltaAngularPosDEC'] = np.empty(shape=[0, 1])
        self.data['status'] = np.empty(shape=[0, 1])
        self.data['sensorWeatherTemp'] = np.empty(shape=[0, 1])
        self.data['sensorWeatherHum'] = np.empty(shape=[0, 1])
        self.data['sensorWeatherPress'] = np.empty(shape=[0, 1])
        self.data['sensorWeatherDew'] = np.empty(shape=[0, 1])
        self.data['onlineWeatherTemp'] = np.empty(shape=[0, 1])
        self.data['onlineWeatherHum'] = np.empty(shape=[0, 1])
        self.data['onlineWeatherPress'] = np.empty(shape=[0, 1])
        self.data['onlineWeatherDew'] = np.empty(shape=[0, 1])
        self.data['directWeatherTemp'] = np.empty(shape=[0, 1])
        self.data['directWeatherHum'] = np.empty(shape=[0, 1])
        self.data['directWeatherPress'] = np.empty(shape=[0, 1])
        self.data['directWeatherDew'] = np.empty(shape=[0, 1])
        self.data['skyTemp'] = np.empty(shape=[0, 1])
        self.data['skySQR'] = np.empty(shape=[0, 1])
        self.data['filterNumber'] = np.empty(shape=[0, 1])
        self.data['focusPosition'] = np.empty(shape=[0, 1])
        self.data['powCurr1'] = np.empty(shape=[0, 1])
        self.data['powCurr2'] = np.empty(shape=[0, 1])
        self.data['powCurr3'] = np.empty(shape=[0, 1])
        self.data['powCurr4'] = np.empty(shape=[0, 1])
        self.data['powVolt'] = np.empty(shape=[0, 1])
        self.data['powCurr'] = np.empty(shape=[0, 1])
        self.data['powHum'] = np.empty(shape=[0, 1])
        self.data['powTemp'] = np.empty(shape=[0, 1])
        self.data['powDew'] = np.empty(shape=[0, 1])
        return True

    def calculateReference(self):
        """
        calculateReference run the states to get the calculation with references
        for RaDec deviations better stable. it takes into account, when the mount
        is tracking and when we calculate the offset (ref) to make the deviations
        balanced to zero

        :return: raJNow, decJNow
        """
        dat = self.data
        obs = self.app.mount.obsSite
        raJNow = 0
        decJNow = 0
        angPosRa = 0
        angPosDec = 0

        if obs.raJNow is None:
            return raJNow, decJNow, angPosRa, angPosDec

        length = len(dat['status'])
        period = min(length, 10)
        hasMean = length > 0 and period > 0

        if not hasMean:
            return raJNow, decJNow, angPosRa, angPosDec

        periodData = dat['status'][-period:]
        hasValidData = all(x is not None for x in periodData)

        if hasValidData:
            trackingIsStable = (periodData.mean() == 0)

        else:
            trackingIsStable = False

        if trackingIsStable:
            if self.raRef is None:
                self.raRef = obs.raJNow._degrees
            if self.decRef is None:
                self.decRef = obs.decJNow.degrees
            if self.angularPosRaRef is None:
                self.angularPosRaRef = obs.angularPosRA.degrees
            if self.angularPosDecRef is None:
                self.angularPosDecRef = obs.angularPosDEC.degrees

            # we would like to have the difference in arcsec
            raJNow = (obs.raJNow._degrees - self.raRef) * 3600
            decJNow = (obs.decJNow.degrees - self.decRef) * 3600

            # we would like to have the difference in arcsec
            angPosRa = (obs.angularPosRA.degrees - self.angularPosRaRef) * 3600
            angPosDec = (obs.angularPosDEC.degrees - self.angularPosDecRef) * 3600

        else:
            self.raRef = None
            self.decRef = None
            self.angularPosRaRef = None
            self.angularPosDecRef = None

        return raJNow, decJNow, angPosRa, angPosDec

    def checkStart(self, lenData):
        """
        checkStart throws the first N measurements away, because they or not valid

        :param lenData:
        :return: True for test purpose
        """

        if self.shorteningStart and lenData > 2:
            self.shorteningStart = False
            for measure in self.data:
                self.data[measure] = np.delete(self.data[measure], range(0, 2))

        return True

    def checkSize(self, lenData):
        """
        checkSize keep tracking of memory usage of the measurement. if the measurement
        get s too much data, it split the history by half and only keeps the latest only
        for work.
        if as well throws the first N measurements away, because they or not valid

        :param lenData:
        :return: True if splitting happens
        """

        if lenData < self.MAXSIZE:
            return False

        for measure in self.data:
            self.data[measure] = np.split(self.data[measure], 2)[1]
        return True

    def getDirectWeather(self):
        """
        getDirectWeather checks if data is already collected and send 0 in case of missing
        data.

        :return: values
        """

        temp = self.app.mount.setting.weatherTemperature
        if temp is None:
            temp = 0
        press = self.app.mount.setting.weatherPressure
        if press is None:
            press = 0
        dew = self.app.mount.setting.weatherDewPoint
        if dew is None:
            dew = 0
        hum = self.app.mount.setting.weatherHumidity
        if hum is None:
            hum = 0

        return temp, press, dew, hum

    def measureTask(self):
        """
        measureTask runs all necessary pre processing and collecting task to assemble a
        large dict of lists, where all measurement data is stored. the intention later on
        would be to store and export this data.
        the time object is related to the time held in mount computer and is in utc timezone.

        data sources are:
            environment
            mount pointing position

        :return: success
        """

        if not self.mutexMeasure.tryLock():
            self.log.debug('overrun in measure')
            return False

        lenData = len(self.data['time'])
        self.checkStart(lenData)
        self.checkSize(lenData)

        dat = self.data

        # gathering all the necessary data
        raJNowD, decJNowD, angularPosRAD, angularPosDECD = self.calculateReference()
        timeStamp = self.app.mount.obsSite.timeJD.utc_datetime().replace(tzinfo=None)
        dat['time'] = np.append(dat['time'], np.datetime64(timeStamp))
        dat['deltaRaJNow'] = np.append(dat['deltaRaJNow'], raJNowD)
        dat['deltaDecJNow'] = np.append(dat['deltaDecJNow'], decJNowD)
        dat['deltaAngularPosRA'] = np.append(dat['deltaAngularPosRA'], angularPosRAD)
        dat['deltaAngularPosDEC'] = np.append(dat['deltaAngularPosDEC'], angularPosDECD)
        dat['status'] = np.append(dat['status'], self.app.mount.obsSite.status)

        sens = self.app.sensorWeather
        sensorWeatherTemp = sens.data.get('WEATHER_PARAMETERS.WEATHER_TEMPERATURE', 0)
        sensorWeatherPress = sens.data.get('WEATHER_PARAMETERS.WEATHER_PRESSURE', 0)
        sensorWeatherDew = sens.data.get('WEATHER_PARAMETERS.WEATHER_DEWPOINT', 0)
        sensorWeatherHum = sens.data.get('WEATHER_PARAMETERS.WEATHER_HUMIDITY', 0)
        dat['sensorWeatherTemp'] = np.append(dat['sensorWeatherTemp'], sensorWeatherTemp)
        dat['sensorWeatherHum'] = np.append(dat['sensorWeatherHum'], sensorWeatherHum)
        dat['sensorWeatherPress'] = np.append(dat['sensorWeatherPress'], sensorWeatherPress)
        dat['sensorWeatherDew'] = np.append(dat['sensorWeatherDew'], sensorWeatherDew)

        onlineWeatherTemp = self.app.onlineWeather.data.get('temperature', 0)
        onlineWeatherPress = self.app.onlineWeather.data.get('pressure', 0)
        onlineWeatherDew = self.app.onlineWeather.data.get('dewPoint', 0)
        onlineWeatherHum = self.app.onlineWeather.data.get('humidity', 0)
        dat['onlineWeatherTemp'] = np.append(dat['onlineWeatherTemp'], onlineWeatherTemp)
        dat['onlineWeatherHum'] = np.append(dat['onlineWeatherHum'], onlineWeatherHum)
        dat['onlineWeatherPress'] = np.append(dat['onlineWeatherPress'], onlineWeatherPress)
        dat['onlineWeatherDew'] = np.append(dat['onlineWeatherDew'], onlineWeatherDew)

        temp, press, dew, hum = self.getDirectWeather()
        dat['directWeatherTemp'] = np.append(dat['directWeatherTemp'], temp)
        dat['directWeatherHum'] = np.append(dat['directWeatherHum'], hum)
        dat['directWeatherPress'] = np.append(dat['directWeatherPress'], press)
        dat['directWeatherDew'] = np.append(dat['directWeatherDew'], dew)

        skySQR = self.app.skymeter.data.get('SKY_QUALITY.SKY_BRIGHTNESS', 0)
        skyTemp = self.app.skymeter.data.get('SKY_QUALITY.SKY_TEMPERATURE', 0)
        dat['skySQR'] = np.append(dat['skySQR'], skySQR)
        dat['skyTemp'] = np.append(dat['skyTemp'], skyTemp)

        filterNo = self.app.filter.data.get('FILTER_SLOT.FILTER_SLOT_VALUE', 0)
        dat['filterNumber'] = np.append(dat['filterNumber'], filterNo)

        focus = self.app.focuser.data.get('ABS_FOCUS_POSITION.FOCUS_ABSOLUTE_POSITION', 0)
        dat['focusPosition'] = np.append(dat['focusPosition'], focus)

        powCurr1 = self.app.power.data.get('POWER_CURRENT.POWER_CURRENT_1', 0)
        powCurr2 = self.app.power.data.get('POWER_CURRENT.POWER_CURRENT_2', 0)
        powCurr3 = self.app.power.data.get('POWER_CURRENT.POWER_CURRENT_3', 0)
        powCurr4 = self.app.power.data.get('POWER_CURRENT.POWER_CURRENT_4', 0)
        powVolt = self.app.power.data.get('POWER_SENSORS.SENSOR_VOLTAGE', 0)
        powCurr = self.app.power.data.get('POWER_SENSORS.SENSOR_CURRENT', 0)
        powTemp = self.app.power.data.get('WEATHER_PARAMETERS.WEATHER_TEMPERATURE', 0)
        powDew = self.app.power.data.get('WEATHER_PARAMETERS.WEATHER_DEWPOINT', 0)
        powHum = self.app.power.data.get('WEATHER_PARAMETERS.WEATHER_HUMIDITY', 0)
        dat['powCurr1'] = np.append(dat['powCurr1'], powCurr1)
        dat['powCurr2'] = np.append(dat['powCurr2'], powCurr2)
        dat['powCurr3'] = np.append(dat['powCurr3'], powCurr3)
        dat['powCurr4'] = np.append(dat['powCurr4'], powCurr4)
        dat['powCurr'] = np.append(dat['powCurr'], powCurr)
        dat['powVolt'] = np.append(dat['powVolt'], powVolt)
        dat['powTemp'] = np.append(dat['powTemp'], powTemp)
        dat['powDew'] = np.append(dat['powDew'], powDew)
        dat['powHum'] = np.append(dat['powHum'], powHum)

        self.mutexMeasure.unlock()

        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries

# external packages
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import requests
import numpy as np
import qimage2ndarray

# local import
from base.tpool import Worker


class Environ(object):
    """
    """

    def __init__(self):
        self.refractionSources = {'onlineWeather': self.ui.onlineWeatherGroup,
                                  'sensorWeather': self.ui.sensorWeatherGroup,
                                  'directWeather': self.ui.directWeatherGroup,
                                  }

        self.refractionSource = ''
        self.filteredTemperature = None
        self.filteredPressure = None

        # environment functions
        signals = self.app.sensorWeather.signals
        signals.deviceDisconnected.connect(self.clearSensorWeatherGui)

        # skymeter functions
        signals = self.app.skymeter.signals
        signals.deviceDisconnected.connect(self.clearSkymeterGui)

        # power weather functions
        signals = self.app.powerWeather.signals
        signals.deviceDisconnected.connect(self.clearPowerWeatherGui)

        # weather functions
        self.app.onlineWeather.signals.dataReceived.connect(self.updateOnlineWeatherGui)

        # weather functions
        self.app.mount.signals.settingDone.connect(self.updateDirectWeatherGui)
        self.app.mount.signals.settingDone.connect(self.updateRefractionUpdateType)

        # gui connections
        self.ui.setRefractionManual.clicked.connect(self.updateRefractionParameters)
        self.ui.isOnline.stateChanged.connect(self.updateClearOutside)
        self.ui.onlineWeatherGroup.clicked.connect(self.selectRefractionSource)
        self.ui.sensorWeatherGroup.clicked.connect(self.selectRefractionSource)
        self.ui.directWeatherGroup.clicked.connect(self.selectRefractionSource)
        self.ui.checkRefracNone.clicked.connect(self.setRefractionUpdateType)
        self.ui.checkRefracCont.clicked.connect(self.setRefractionUpdateType)
        self.ui.checkRefracNoTrack.clicked.connect(self.setRefractionUpdateType)

        # cyclic functions
        self.app.update1s.connect(self.updateFilterRefractionParameters)
        self.app.update1s.connect(self.updateRefractionParameters)
        self.app.update1s.connect(self.updateSkymeterGui)
        self.app.update1s.connect(self.updatePowerWeatherGui)
        self.app.update1s.connect(self.updateSensorWeatherGui)
        self.app.update30m.connect(self.updateClearOutside)

    def initConfig(self):
        """
        :return: True for test purpose
        """
        config = self.app.config['mainW']
        self.ui.checkRefracNone.setChecked(config.get('checkRefracNone', False))
        self.ui.checkRefracCont.setChecked(config.get('checkRefracCont', False))
        self.ui.checkRefracNoTrack.setChecked(config.get('checkRefracNoTrack', False))

        self.refractionSource = config.get('refractionSource', '')
        self.setRefractionSourceGui()
        self.updateClearOutside()
        return True

    def storeConfig(self):
        """
        :return: True for test purpose
        """
        config = self.app.config['mainW']
        config['checkRefracNone'] = self.ui.checkRefracNone.isChecked()
        config['checkRefracCont'] = self.ui.checkRefracCont.isChecked()
        config['checkRefracNoTrack'] = self.ui.checkRefracNoTrack.isChecked()
        config['refractionSource'] = self.refractionSource
        return True

    def updateRefractionUpdateType(self, setting):
        """
        :param setting:
        :return: success
        """
        if self.refractionSource != 'directWeather':
            return False

        if setting.weatherStatus == 0:
            self.ui.checkRefracNone.setChecked(True)
        elif setting.weatherStatus == 1:
            self.ui.checkRefracNoTrack.setChecked(True)
        elif setting.weatherStatus == 2:
            self.ui.checkRefracCont.setChecked(True)
        else:
            return False

        return True

    def setRefractionUpdateType(self):
        """
        :return: success
        """
        if self.refractionSource != 'directWeather':
            suc = self.app.mount.setting.setDirectWeatherUpdateType(0)
            return suc

        # otherwise we have to switch it on or off
        if self.ui.checkRefracNone.isChecked():
            suc = self.app.mount.setting.setDirectWeatherUpdateType(0)
        elif self.ui.checkRefracNoTrack.isChecked():
            suc = self.app.mount.setting.setDirectWeatherUpdateType(1)
        else:
            suc = self.app.mount.setting.setDirectWeatherUpdateType(2)

        return suc

    def setRefractionSourceGui(self):
        """
        setRefractionSourceGui sets the gui elements to a recognizable setting
        and disables all others

        :return: success
        """
        for source, group in self.refractionSources.items():
            if self.refractionSource == source:
                self.changeStyleDynamic(group, 'refraction', True)
                group.setChecked(True)
            else:
                self.changeStyleDynamic(group, 'refraction', False)
                group.setChecked(False)
        return True

    def selectRefractionSource(self):
        """
        selectRefractionSource receives all button presses on groups and checks
        which of the groups was clicked on. whit that information is detects the
        index in the list of groups.

        :return: success
        """
        old = self.refractionSource

        for source, group in self.refractionSources.items():
            if group != self.sender():
                continue
            if group.isChecked():
                self.refractionSource = source
            else:
                self.refractionSource = ''

        if old != self.refractionSource:
            self.filteredTemperature = None
            self.filteredPressure = None

        self.setRefractionSourceGui()
        self.setRefractionUpdateType()

        return True

    def updateFilterRefractionParameters(self):
        """
        updateFilter initializes the filter with the first values or is rolling
        the moving average

        :return:
        """
        if self.refractionSource == 'onlineWeather':
            if not self.app.onlineWeather.data:
                return False
            temp = self.app.onlineWeather.data['temperature']
            press = self.app.onlineWeather.data['pressure']

        elif self.refractionSource == 'sensorWeather':
            key = 'WEATHER_PARAMETERS.WEATHER_TEMPERATURE'
            temp = self.app.sensorWeather.data.get(key, None)
            key = 'WEATHER_PARAMETERS.WEATHER_PRESSURE'
            press = self.app.sensorWeather.data.get(key, None)

        else:
            temp = None
            press = None

        if temp is None or press is None:
            self.filteredTemperature = None
            self.filteredPressure = None
            return False

        if self.filteredTemperature is None:
            self.filteredTemperature = np.full(100, temp)

        else:
            self.filteredTemperature = np.roll(self.filteredTemperature, 1)
            self.filteredTemperature[0] = temp

        if self.filteredPressure is None:
            self.filteredPressure = np.full(100, press)

        else:
            self.filteredPressure = np.roll(self.filteredPressure, 1)
            self.filteredPressure[0] = press

        return True

    def movingAverageRefractionParameters(self):
        """
        getFilteredRefracParams filters local temperature and pressure with and
        moving average filter over 100 seconds and returns the filtered values.

        :return:  temperature and pressure
        """
        if self.filteredTemperature is None or self.filteredPressure is None:
            return None, None

        temp = np.mean(self.filteredTemperature)
        press = np.mean(self.filteredPressure)
        return temp, press

    def updateRefractionParameters(self):
        """
        updateRefractionParameters takes the actual conditions for update into
        account and does the update of the refraction parameters. this could be
        done during when mount is not in tracking state or continuously

        :return: success if update happened
        """
        if self.refractionSource == 'directWeather':
            return False

        if not self.deviceStat['mount']:
            return False

        temp, press = self.movingAverageRefractionParameters()
        if temp is None or press is None:
            return False

        if self.sender() != self.ui.setRefractionManual:
            if self.ui.checkRefracNone.isChecked():
                return False

            if self.ui.checkRefracNoTrack.isChecked():
                if self.app.mount.obsSite.status == 0:
                    return False

        suc = self.app.mount.setting.setRefractionParam(temperature=temp,
                                                        pressure=press)

        if not suc:
            self.app.message.emit('Cannot perform refraction update', 2)
            return False

        return True

    def clearSensorWeatherGui(self, deviceName):
        """
        :param deviceName:
        :return: true for test purpose
        """
        self.ui.sensorWeatherTemp.setText('-')
        self.ui.sensorWeatherPress.setText('-')
        self.ui.sensorWeatherDewPoint.setText('-')
        self.ui.sensorWeatherHumidity.setText('-')
        self.ui.sensorWeatherCloudCover.setText('-')
        self.ui.sensorWeatherRainVol.setText('-')
        self.ui.sensorWeatherWindSpeed.setText('-')
        self.ui.sensorWeatherWindDir.setText('-')
        self.ui.sensorWeatherSQR.setText('-')
        return True

    def updateSensorWeatherGui(self):
        """
        :return:    True if ok for testing
        """
        value = self.app.sensorWeather.data.get('WEATHER_PARAMETERS.WEATHER_TEMPERATURE', 0)
        self.guiSetText(self.ui.sensorWeatherTemp, '4.1f', value)

        value = self.app.sensorWeather.data.get('WEATHER_PARAMETERS.WEATHER_PRESSURE', 0)
        self.guiSetText(self.ui.sensorWeatherPress, '4.1f', value)

        value = self.app.sensorWeather.data.get('WEATHER_PARAMETERS.WEATHER_DEWPOINT', 0)
        self.guiSetText(self.ui.sensorWeatherDewPoint, '4.1f', value)

        value = self.app.sensorWeather.data.get('WEATHER_PARAMETERS.WEATHER_HUMIDITY', 0)
        self.guiSetText(self.ui.sensorWeatherHumidity, '3.0f', value)

        value = self.app.sensorWeather.data.get('cloudCover', 0)
        self.guiSetText(self.ui.sensorWeatherCloudCover, '3.0f', value)

        value = self.app.sensorWeather.data.get('rain', 0)
        self.guiSetText(self.ui.sensorWeatherRainVol, '5.2f', value)

        value = self.app.sensorWeather.data.get('windDir', 0)
        self.guiSetText(self.ui.sensorWeatherWindDir, '3.0f', value)

        value = self.app.sensorWeather.data.get('windSpeed', 0)
        self.guiSetText(self.ui.sensorWeatherWindSpeed, '3.0f', value)

        value = self.app.sensorWeather.data.get('SKY_QUALITY.SKY_BRIGHTNESS', 0)
        self.guiSetText(self.ui.sensorWeatherSQR, '4.2f', value)
        return True

    def clearSkymeterGui(self, deviceName=''):
        """
        :param deviceName:
        :return: true for test purpose
        """
        self.ui.skymeterSQR.setText('-')
        self.ui.skymeterTemp.setText('-')
        return True

    def updateSkymeterGui(self):
        """
        :return:    True if ok for testing
        """
        value = self.app.skymeter.data.get('SKY_QUALITY.SKY_BRIGHTNESS', 0)
        self.guiSetText(self.ui.skymeterSQR, '5.2f', value)

        value = self.app.skymeter.data.get('SKY_QUALITY.SKY_TEMPERATURE', 0)
        self.guiSetText(self.ui.skymeterTemp, '4.1f', value)
        return True

    def clearPowerWeatherGui(self):
        """
        clearPowerWeatherGui changes the state of the Pegasus values to '-'

        :return: success for test
        """

        self.ui.powerTemp.setText('-')
        self.ui.powerHumidity.setText('-')
        self.ui.powerDewPoint.setText('-')

        return True

    def updatePowerWeatherGui(self):
        """
        updatePowerGui changes the style of the button related to the state of
        the Pegasus UPB device

        :return: success for test
        """

        value = self.app.powerWeather.data.get('WEATHER_PARAMETERS.WEATHER_TEMPERATURE', 0)
        self.ui.powerTemp.setText('{0:4.1f}'.format(value))
        value = self.app.powerWeather.data.get('WEATHER_PARAMETERS.WEATHER_HUMIDITY', 0)
        self.ui.powerHumidity.setText('{0:3.0f}'.format(value))
        value = self.app.powerWeather.data.get('WEATHER_PARAMETERS.WEATHER_DEWPOINT', 0)
        self.ui.powerDewPoint.setText('{0:4.1f}'.format(value))

        return True

    def getWebDataWorker(self, url=''):
        """
        :param url:
        :return: data
        """
        if not url:
            return None

        try:
            data = requests.get(url, timeout=30)

        except Exception as e:
            self.log.critical(f'{url} general exception: {e}')
            return None

        if data.status_code != 200:
            self.log.warning(f'{url}: status nok')
            return None

        self.log.trace(f'{url}: {data.status_code}')
        return data

    @staticmethod
    def processClearOutsideImage(image=None):
        """
        processClearOutsideImage takes the image, split it and puts the image
        to the Gui. for the transformation qimage2ndarray is used because of
        the speed for the calculations. dim is a factor which reduces the
        lightness of the overall image

        :param image:
        :return: success
        """
        dim = 0.85
        image.convertToFormat(PyQt5.QtGui.QImage.Format_RGB32)
        imageBase = image.copy(0, 84, 624, 141)

        # transformation are done in numpy, because it's much faster
        width = imageBase.width()
        height = imageBase.height()
        imgArr = qimage2ndarray.rgb_view(imageBase)
        imgArr = imgArr.reshape(width * height, 3)
        img_Max = np.maximum(255 - imgArr, [32, 32, 32])
        temp = imgArr[:, 0] == imgArr[:, 1]
        check = np.array([temp, temp, temp]).transpose()

        # do the transform light to dark theme
        imgArr = np.where(check, img_Max, imgArr)

        # transforming back
        imgArr = imgArr.reshape(height, width, 3)

        # Compressing the image as the widget content is a png
        # removing some rows
        m = np.isin(imgArr, [[32, 32, 32], [255, 0, 0]])
        toDelete = []
        maxRow = 1
        line = maxRow
        for i in range(0, len(m[:, 1])):
            if not line and m[i, :].all() and i > 15:
                toDelete.append(i)
            elif line:
                line -= 1
            elif not m[i][:].all():
                line = maxRow
        imgArr = np.delete(imgArr, toDelete, axis=0)

        # re transfer to QImage from numpy array
        imageBase = qimage2ndarray.array2qimage(dim * imgArr)
        pixmapBase = PyQt5.QtGui.QPixmap().fromImage(imageBase)

        return pixmapBase

    def updateClearOutsideImage(self, data=None):
        """
        updateClearOutsideImage takes the returned data from a web fetch and
        makes an image out of it

        :param data:
        :return: success
        """
        if data is None:
            return False

        image = PyQt5.QtGui.QImage()
        if not hasattr(data, 'content'):
            return False
        if not isinstance(data.content, bytes):
            return False

        image.loadFromData(data.content)
        pixmapBase = self.processClearOutsideImage(image=image)
        self.ui.picClearOutside.setPixmap(pixmapBase)
        return True

    def getClearOutside(self, url=''):
        """
        getClearOutside initiates the worker thread to get the web data fetched

        :param url:
        :return:
        """
        worker = Worker(self.getWebDataWorker, url)
        worker.signals.result.connect(self.updateClearOutsideImage)
        self.threadPool.start(worker)

    def updateClearOutside(self):
        """
        updateClearOutside downloads the actual clear outside image and displays
        it in environment tab. it checks first if online is set, otherwise not
        download will take place. it will be updated every 30 minutes.

        confirmation for using the service :

        Grant replied Aug 5, 10:01am
        Hi Michael,
        No problem at all embedding the forecast as shown in your image :-)
        We appreciate the support.
        Kindest Regards,
        Grant

        :return: success
        """
        if not self.ui.isOnline.isChecked():
            pixmap = PyQt5.QtGui.QPixmap(':/pics/offlineMode.png')
            self.ui.picClearOutside.setPixmap(pixmap)
            return False

        # prepare coordinates for website
        loc = self.app.mount.obsSite.location
        lat = loc.latitude.degrees
        lon = loc.longitude.degrees

        webSite = 'http://clearoutside.com/forecast_image_medium/'
        url = f'{webSite}{lat:4.2f}/{lon:4.2f}/forecast.png'
        self.getClearOutside(url=url)
        return True

    def clearOnlineWeatherGui(self):
        """
        :return: true for test purpose
        """
        self.ui.onlineWeatherTemp.setText('-')
        self.ui.onlineWeatherPress.setText('-')
        self.ui.onlineWeatherHumidity.setText('-')
        self.ui.onlineWeatherDewPoint.setText('-')
        self.ui.onlineWeatherCloudCover.setText('-')
        self.ui.onlineWeatherWindSpeed.setText('-')
        self.ui.onlineWeatherWindDir.setText('-')
        self.ui.onlineWeatherRainVol.setText('-')
        return True

    def updateOnlineWeatherGui(self, data=None):
        """
        :return: success
        """
        if not data:
            self.clearOnlineWeatherGui()
            return False

        if 'temperature' in data:
            self.ui.onlineWeatherTemp.setText(f'{data["temperature"]:4.1f}')
        if 'pressure' in data:
            self.ui.onlineWeatherPress.setText(f'{data["pressure"]:5.1f}')
        if 'humidity' in data:
            self.ui.onlineWeatherHumidity.setText(f'{data["humidity"]:3.0f}')
        if 'dewPoint' in data:
            self.ui.onlineWeatherDewPoint.setText(f'{data["dewPoint"]:4.1f}')
        if 'cloudCover' in data:
            self.ui.onlineWeatherCloudCover.setText(f'{data["cloudCover"]:3.0f}')
        if 'windSpeed' in data:
            self.ui.onlineWeatherWindSpeed.setText(f'{data["windSpeed"]:3.0f}')
        if 'windDir' in data:
            self.ui.onlineWeatherWindDir.setText(f'{data["windDir"]:3.0f}')
        if 'rain' in data:
            self.ui.onlineWeatherRainVol.setText(f'{data["rain"]:5.2f}')
        return True

    def clearDirectWeatherGui(self):
        """
        updateOnlineWeatherGui takes the returned data from the dict to the Gui

        :return: true for test purpose
        """
        self.ui.directWeatherTemp.setText('-')
        self.ui.directWeatherPress.setText('-')
        self.ui.directWeatherHumidity.setText('-')
        self.ui.directWeatherDewPoint.setText('-')
        return True

    def updateDirectWeatherGui(self, setting=None):
        """
        :param setting:
        :return: success
        """
        if not self.deviceStat['directWeather']:
            self.clearDirectWeatherGui()
            return False

        if setting is None:
            return False

        if setting.weatherTemperature is not None:
            self.ui.directWeatherTemp.setText(f'{setting.weatherTemperature:4.1f}')
        if setting.weatherPressure is not None:
            self.ui.directWeatherPress.setText(f'{setting.weatherPressure:5.1f}')
        if setting.weatherHumidity is not None:
            self.ui.directWeatherHumidity.setText(f'{setting.weatherHumidity:3.0f}')
        if setting.weatherDewPoint is not None:
            self.ui.directWeatherDewPoint.setText(f'{setting.weatherDewPoint:4.1f}')
        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries

# external packages
import numpy as np
from PyQt5.QtGui import QFont
from PyQt5.QtGui import QVector3D
from PyQt5.Qt3DExtras import QSphereMesh
from PyQt5.Qt3DExtras import QExtrudedTextMesh, QCylinderMesh
from PyQt5.Qt3DCore import QEntity, QTransform
from skyfield import functions

# local import
from gui.extWindows.simulator.materials import Materials


class SimulatorBuildPoints:

    __all__ = ['SimulatorBuildPoints',
               ]

    def __init__(self, app):
        self.app = app
        self.points = []
        self.pointRoot = None

    @staticmethod
    def createLine(rEntity, dx, dy, dz):
        """
        create line draw a line between two point or better along dx, dy, dz. Therefore three
        transformations are made and the resulting vector has to be translated half the
        length, because is will be drawn symmetrically to the starting point.

        :param rEntity:
        :param dx:
        :param dy:
        :param dz:
        :return:
        """

        radius, alt, az = functions.to_spherical(np.array((dx, dy, dz)))
        az = np.degrees(az)
        alt = np.degrees(alt)

        e1 = QEntity(rEntity)
        trans1 = QTransform()
        trans1.setRotationZ(az + 90)
        e1.addComponent(trans1)

        e2 = QEntity(e1)
        trans2 = QTransform()
        trans2.setRotationX(-alt)
        e2.addComponent(trans2)

        e3 = QEntity(e2)
        mesh = QCylinderMesh()
        mesh.setRadius(0.008)
        mesh.setLength(radius)
        trans3 = QTransform()
        trans3.setTranslation(QVector3D(0, radius / 2, 0))
        e3.addComponent(mesh)
        e3.addComponent(trans3)
        e3.addComponent(Materials().lines)

        return e3

    @staticmethod
    def createPoint(rEntity, alt, az):
        """
        the point is located in a distance of radius meters from the ota axis and
        positioned in azimuth and altitude correctly. it's representation is a small
        small ball mesh.

        :param rEntity:
        :param alt:
        :param az:
        :return: entity, x, y, z coordinates
        """

        radius = 4
        entity = QEntity(rEntity)
        mesh = QSphereMesh()
        mesh.setRadius(0.035)
        mesh.setRings(30)
        mesh.setSlices(30)
        trans = QTransform()
        x, y, z = functions.from_spherical(radius, alt, az)
        trans.setTranslation(QVector3D(x, y, z + 1.35))
        entity.addComponent(mesh)
        entity.addComponent(trans)
        entity.addComponent(Materials().points)

        return entity, x, y, z

    @staticmethod
    def createAnnotation(rEntity, alt, az, text, faceIn=False):
        """
        the annotation - basically the number of the point - is positioned relative to the
        build point in its local coordinate system. to face the text to the viewer (azimuth),
        the text is first rotated to be upright and secondly to turn is fac according to
        the altitude of the viewer.
        it is done in two entities to simplify the rotations as they are in this case
        relative to each other.
        faceIn changes the behaviour to have the text readable from inside or outside.

        :param rEntity:
        :param alt:
        :param az:
        :param text:
        :param faceIn: direction of the text face (looking from inside or outside)
        :return: entity
        """

        e1 = QEntity(rEntity)
        trans1 = QTransform()
        if faceIn:
            trans1.setRotationZ(az - 90)
        else:
            trans1.setRotationZ(az + 90)
        e1.addComponent(trans1)

        e2 = QEntity(e1)
        mesh = QExtrudedTextMesh()
        mesh.setText(text)
        mesh.setDepth(0.05)
        mesh.setFont(QFont('Arial', 36))
        trans2 = QTransform()
        if faceIn:
            trans2.setRotationX(90 + alt)
        else:
            trans2.setRotationX(90 - alt)
        trans2.setScale(0.12)
        e2.addComponent(mesh)
        e2.addComponent(trans2)
        e2.addComponent(Materials().numbers)

        return e2

    def create(self, rEntity, show, numbers=False, path=False):
        """
        buildPointsCreate show the point in the sky if checked, in addition if selected the
        slew path between the points and in addition if checked the point numbers
        as the azimuth (second element in tuple) is turning clockwise, it's opposite to the
        right turning coordinate system (z is upwards), which means angle around z
        (which is azimuth) turns counterclockwise. so we have to set - azimuth for coordinate
        calculation

        :return: success
        """

        if self.points:
            self.pointRoot.setParent(None)

        self.points.clear()

        if not show:
            return False

        if not self.app.data.buildP:
            return False

        self.pointRoot = QEntity(rEntity)

        for index, point in enumerate(self.app.data.buildP):
            e, x, y, z = self.createPoint(self.pointRoot,
                                          np.radians(point[0]),
                                          np.radians(-point[1]))

            if numbers:
                a = self.createAnnotation(e, point[0], -point[1], f'{index:02d}')
            else:
                a = None

            if index and path:
                x0 = self.points[-1]['x']
                y0 = self.points[-1]['y']
                z0 = self.points[-1]['z']
                dx = x - x0
                dy = y - y0
                dz = z - z0
                li = self.createLine(e, dx, dy, dz)
            else:
                li = None

            element = {'e': e, 'a': a, 'li': li, 'x': x, 'y': y, 'z': z}

            self.points.append(element)

        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
import platform

# external packages
from PyQt5.QtCore import QObject, pyqtSignal

# local imports
from logic.cover.coverIndi import CoverIndi
from logic.cover.coverAlpaca import CoverAlpaca
if platform.system() == 'Windows':
    from logic.cover.coverAscom import CoverAscom


class CoverSignals(QObject):
    """
    The FlipFlatSignals class offers a list of signals to be used and instantiated by
    the Mount class to get signals for triggers for finished tasks to
    enable a gui to update their values transferred to the caller back.

    This has to be done in a separate class as the signals have to be subclassed from
    QObject and the Mount class itself is subclassed from object
    """

    __all__ = ['CoverSignals']

    serverConnected = pyqtSignal()
    serverDisconnected = pyqtSignal(object)
    deviceConnected = pyqtSignal(str)
    deviceDisconnected = pyqtSignal(str)


class Cover:

    __all__ = ['Cover',
               ]

    log = logging.getLogger(__name__)

    def __init__(self, app):

        self.app = app
        self.threadPool = app.threadPool
        self.signals = CoverSignals()

        self.data = {}
        self.defaultConfig = {'framework': '',
                              'frameworks': {}}
        self.framework = ''
        self.run = {
            'indi': CoverIndi(self.app, self.signals, self.data),
            'alpaca': CoverAlpaca(self.app, self.signals, self.data),
        }

        if platform.system() == 'Windows':
            self.run['ascom'] = CoverAscom(self.app, self.signals, self.data)
            ascomSignals = self.run['ascom'].ascomSignals
            ascomSignals.serverConnected.connect(self.signals.serverConnected)
            ascomSignals.serverDisconnected.connect(self.signals.serverDisconnected)
            ascomSignals.deviceConnected.connect(self.signals.deviceConnected)
            ascomSignals.deviceDisconnected.connect(self.signals.deviceDisconnected)

        for fw in self.run:
            self.defaultConfig['frameworks'].update(self.run[fw].defaultConfig)

        # signalling from subclasses to main
        alpacaSignals = self.run['alpaca'].client.signals
        alpacaSignals.serverConnected.connect(self.signals.serverConnected)
        alpacaSignals.serverDisconnected.connect(self.signals.serverDisconnected)
        alpacaSignals.deviceConnected.connect(self.signals.deviceConnected)
        alpacaSignals.deviceDisconnected.connect(self.signals.deviceDisconnected)

        # signalling from subclasses to main
        indiSignals = self.run['indi'].client.signals
        indiSignals.serverConnected.connect(self.signals.serverConnected)
        indiSignals.serverDisconnected.connect(self.signals.serverDisconnected)
        indiSignals.deviceConnected.connect(self.signals.deviceConnected)
        indiSignals.deviceDisconnected.connect(self.signals.deviceDisconnected)

    def startCommunication(self, loadConfig=False):
        """
        :param loadConfig:
        :return:
        """
        if self.framework not in self.run.keys():
            return False
        suc = self.run[self.framework].startCommunication(loadConfig=loadConfig)
        return suc

    def stopCommunication(self):
        """
        :return:
        """
        if self.framework not in self.run.keys():
            return False

        suc = self.run[self.framework].stopCommunication()
        return suc

    def closeCover(self):
        """
        :return: success
        """
        if self.framework not in self.run.keys():
            return False

        suc = self.run[self.framework].closeCover()
        return suc

    def openCover(self):
        """
        :return: success
        """
        if self.framework not in self.run.keys():
            return False

        suc = self.run[self.framework].openCover()
        return suc

    def haltCover(self):
        """
        :return: success
        """
        if self.framework not in self.run.keys():
            return False

        suc = self.run[self.framework].haltCover()
        return suc



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
import subprocess
import os
import glob
import time
import platform

# external packages
from astropy.io import fits

# local imports
from mountcontrol import convert


class AstrometryNET(object):
    """
    the class Astrometry inherits all information and handling of astrometry.net handling

    Keyword definitions could be found under
        https://fits.gsfc.nasa.gov/fits_dictionary.html

        >>> astrometry = AstrometryNET(app=app,
        >>>                         )

    """

    __all__ = ['AstrometryNET',
               ]

    log = logging.getLogger(__name__)

    def __init__(self, parent=None):
        self.parent = parent
        self.data = parent.data
        self.tempDir = parent.tempDir
        self.readFitsData = parent.readFitsData
        self.getSolutionFromWCS = parent.getSolutionFromWCS

        self.result = {'success': False}
        self.process = None
        self.apiKey = ''
        self.indexPath = ''
        self.appPath = ''
        self.timeout = 30
        self.searchRadius = 20
        self.deviceName = 'ASTROMETRY.NET'

        self.setDefaultPath()

        self.defaultConfig = {
            'astrometry': {
                'deviceName': 'ASTROMETRY.NET',
                'deviceList': ['ASTROMETRY.NET'],
                'searchRadius': 10,
                'timeout': 30,
                'appPath': self.appPath,
                'indexPath': self.indexPath,
            }
        }

    def setDefaultPath(self):
        """

        :return: true for test purpose
        """

        if platform.system() == 'Darwin':
            home = os.environ.get('HOME', '')
            self.appPath = '/Applications/KStars.app/Contents/MacOS/astrometry/bin'
            self.indexPath = home + '/Library/Application Support/Astrometry'

        elif platform.system() == 'Linux':
            self.appPath = '/usr/bin'
            self.indexPath = '/usr/share/astrometry'

        elif platform.system() == 'Windows':
            self.appPath = ''
            self.indexPath = ''

        return True

    def runImage2xy(self, binPath='', tempPath='', fitsPath=''):
        """
        runImage2xy extracts a list of stars out of the fits image. there is a timeout of
        3 seconds set to get the process finished

        :param binPath:   full path to image2xy executable
        :param tempPath:  full path to star file
        :param fitsPath:  full path to fits file
        :return: success
        """

        runnable = [binPath,
                    '-O',
                    '-o',
                    tempPath,
                    fitsPath]

        timeStart = time.time()
        try:
            self.process = subprocess.Popen(args=runnable,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE,
                                            )
            stdout, stderr = self.process.communicate(timeout=self.timeout)

        except subprocess.TimeoutExpired as e:
            self.log.critical(e)
            return False

        except Exception as e:
            self.log.critical(f'error: {e} happened')
            return False

        else:
            delta = time.time() - timeStart
            self.log.debug(f'IMAGE2XY took {delta}s return code: '
                           + str(self.process.returncode)
                           + f' [{fitsPath}]'
                           + ' stderr: '
                           + stderr.decode().replace('\n', ' ')
                           + ' stdout: '
                           + stdout.decode().replace('\n', ' ')
                           )

        success = (self.process.returncode == 0)
        return success

    def runSolveField(self, binPath='', configPath='', tempPath='', options='', fitsPath=''):
        """
        runSolveField solves finally the xy star list and writes the WCS data in a fits
        file format

        :param binPath:   full path to image2xy executable
        :param configPath: full path to astrometry.cfg file
        :param tempPath:  full path to star file
        :param options: additional solver options e.g. ra and dec hint
        :param fitsPath:  full path to fits file
        :return: success
        """

        runnable = [binPath,
                    '--overwrite',
                    '--no-remove-lines',
                    '--no-plots',
                    '--no-verify-uniformize',
                    '--uniformize', '0',
                    '--sort-column', 'FLUX',
                    '--scale-units', 'app',
                    '--crpix-center',
                    '--cpulimit', str(self.timeout),
                    '--config',
                    configPath,
                    tempPath,
                    ]

        runnable += options

        timeStart = time.time()
        try:
            self.process = subprocess.Popen(args=runnable,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE,
                                            )
            stdout, stderr = self.process.communicate(timeout=self.timeout)

        except subprocess.TimeoutExpired as e:
            self.log.critical(e)
            return False

        except Exception as e:
            self.log.critical(f'error: {e} happened')
            return False

        else:
            delta = time.time() - timeStart
            self.log.debug(f'SOLVE-FIELD took {delta}s return code: '
                           + str(self.process.returncode)
                           + f' [{fitsPath}]'
                           + ' stderr: '
                           + stderr.decode().replace('\n', ' ')
                           + ' stdout: '
                           + stdout.decode().replace('\n', ' ')
                           )

        success = (self.process.returncode == 0)

        return success

    @staticmethod
    def getWCSHeader(wcsHDU=None):
        """
        getWCSHeader returns the header part of a fits HDU

        :param wcsHDU: fits file with wcs data
        :return: wcsHeader
        """
        if wcsHDU is None:
            return None

        wcsHeader = wcsHDU[0].header
        return wcsHeader

    def solve(self, fitsPath='', raHint=None, decHint=None, scaleHint=None,
              updateFits=False):
        """
        Solve uses the astrometry.net solver capabilities. The intention is to use an
        offline solving capability, so we need a installed instance. As we go multi
        platform and we need to focus on MW function, we use the astrometry.net package
        which is distributed with KStars / EKOS. Many thanks to them providing such a
        nice package.
        As we go using astrometry.net we focus on the minimum feature set possible to
        omit many of the installation and wrapping work to be done. So we only support
        solving of FITS files, use no python environment for astrometry.net parts (as we
        could access these via MW directly)

        The base outside ideas of implementation come from astrometry.net itself and the
        astrometry implementation from cloudmakers.eu (another nice package for MAC Astro
        software)

        :param fitsPath:  full path to fits file
        :param raHint:  ra dest to look for solve in J2000
        :param decHint:  dec dest to look for solve in J2000
        :param scaleHint:  scale to look for solve in J2000
        :param updateFits:  if true update Fits image file with wcsHeader data

        :return: success
        """

        self.process = None
        self.result = {'success': False}

        if not os.path.isfile(fitsPath):
            self.result['message'] = 'image missing'
            return False

        tempPath = self.tempDir + '/temp.xy'
        configPath = self.tempDir + '/astrometry.cfg'
        solvedPath = self.tempDir + '/temp.solved'
        wcsPath = self.tempDir + '/temp.wcs'
        binPathImage2xy = self.appPath + '/image2xy'
        binPathSolveField = self.appPath + '/solve-field'

        if os.path.isfile(wcsPath):
            os.remove(wcsPath)

        cfgFile = self.tempDir + '/astrometry.cfg'
        with open(cfgFile, 'w+') as outFile:
            outFile.write('cpulimit 300\n')
            outFile.write(f'add_path {self.indexPath}\n')
            outFile.write('autoindex\n')

        suc = self.runImage2xy(binPath=binPathImage2xy,
                               tempPath=tempPath,
                               fitsPath=fitsPath,
                               )
        if not suc:
            self.log.warning(f'IMAGE2XY error in [{fitsPath}]')
            self.result['message'] = 'image2xy failed'
            return False

        raFITS, decFITS, scaleFITS, _, _ = self.readFitsData(fitsPath=fitsPath)

        # if parameters are passed, they have priority
        if raHint is None:
            raHint = raFITS

        if decHint is None:
            decHint = decFITS

        if scaleHint is None:
            scaleHint = scaleFITS

        searchRatio = 1.1
        ra = convert.convertToHMS(raHint)
        dec = convert.convertToDMS(decHint)
        scaleLow = scaleHint / searchRatio
        scaleHigh = scaleHint * searchRatio
        options = ['--scale-low',
                   f'{scaleLow}',
                   '--scale-high',
                   f'{scaleHigh}',
                   '--ra',
                   f'{ra}',
                   '--dec',
                   f'{dec}',
                   '--radius',
                   f'{self.searchRadius:1.1f}',
                   ]

        # split between ekos and cloudmakers as cloudmakers use an older version of
        # solve-field, which need the option '--no-fits2fits', whereas the actual
        # version used in KStars throws an error using this option.
        if 'Astrometry.app' in self.appPath:
            options.append('--no-fits2fits')

        suc = self.runSolveField(binPath=binPathSolveField,
                                 configPath=configPath,
                                 tempPath=tempPath,
                                 options=options,
                                 fitsPath=fitsPath,
                                 )
        if not suc:
            self.log.warning(f'SOLVE-FIELD error in [{fitsPath}]')
            self.result['message'] = 'solve-field error'
            return False

        if not os.path.isfile(solvedPath):
            self.log.debug(f'Solve files for [{fitsPath}] missing')
            self.result['message'] = 'solve failed'
            return False

        if not os.path.isfile(wcsPath):
            self.log.debug(f'Solve files for [{wcsPath}] missing')
            self.result['message'] = 'solve failed'
            return False

        with fits.open(wcsPath) as wcsHDU:
            wcsHeader = self.getWCSHeader(wcsHDU=wcsHDU)

        with fits.open(fitsPath, mode='update') as fitsHDU:
            solve, header = self.getSolutionFromWCS(fitsHeader=fitsHDU[0].header,
                                                    wcsHeader=wcsHeader,
                                                    updateFits=updateFits)
            fitsHDU[0].header = header

        self.result = {
            'success': True,
            'solvedPath': fitsPath,
            'message': 'Solved',
        }
        self.result.update(solve)

        return True

    def abort(self):
        """
        abortNET stops the solving function hardly just by killing the process

        :return: success
        """

        if self.process:
            self.process.kill()
            return True
        else:
            return False

    def checkAvailability(self):
        """
        :return: working environment found
        """
        if platform.system() == 'Darwin':
            program = self.appPath + '/solve-field'
            index = self.indexPath + '/*.fits'
        elif platform.system() == 'Linux':
            program = self.appPath + '/solve-field'
            index = self.indexPath + '/*.fits'
        elif platform.system() == 'Windows':
            program = ''
            index = ''

        # checking binaries
        if not os.path.isfile(program):
            self.log.info(f'[{program}] not found')
            sucProgram = False
        else:
            sucProgram = True

        # checking indexes
        if not glob.glob(index):
            self.log.info('No index files found')
            sucIndex = False
        else:
            sucIndex = True

        self.log.info(f'astrometry.net OK, app:{program} index:{index}')

        return sucProgram, sucIndex



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import socket

# external packages
from PyQt5.QtCore import QMutex, QTimer, QThreadPool

# local import
from base.tpool import Worker
from indibase.indiBase import Client


class Client(Client):
    """
    Client implements an INDI Base Client for INDI servers. it rely on PyQt5 and it's
    signalling scheme. there might be not all capabilities implemented right now. all
    the data, properties and attributes are stored in a the devices dict.
    The reading and parsing of the XML data is done in a streaming way, so for xml the
    xml.parse.feed() mechanism is used.

        >>> indiClient = Client(
        >>>                     host=host
        >>>                     )

    """

    __all__ = ['Client',
               ]

    SOCKET_TIMEOUT = 3
    CYCLE_SERVER_UP = 5000

    def __init__(self, host=None, threadPool=None):
        super().__init__(host=host)
        if threadPool is None:
            self.threadPool = QThreadPool()

        else:
            self.threadPool = threadPool

        self.mutexServerUp = QMutex()
        self.timerServerUp = QTimer()
        self.timerServerUp.setSingleShot(False)
        self.timerServerUp.timeout.connect(self.cycleCheckServerUp)

    def checkServerUp(self):
        """
        checkServerUp polls the host/port of the mount computer and set the state and
        signals for the status accordingly.

        :return: nothing
        """
        client = socket.socket()
        client.settimeout(self.SOCKET_TIMEOUT)
        try:
            client.connect(self.host)

        except Exception:
            return False

        else:
            return True

        finally:
            # client.shutdown(socket.SHUT_RDWR)
            client.close()

    def checkServerUpResult(self, result):
        """
        :param result:
        :return:
        """
        if result and not self.connected:
            suc = self.connectServer()
            self.log.info(f'Connect to server, result: {suc}')
            return suc

        else:
            return False

    def errorCycleCheckServerUp(self, e):
        """
        :return: nothing
        """
        self.log.error(f'Cycle error: {e}')
        return True

    def clearCycleCheckServerUp(self):
        """
        :return: nothing
        """
        self.mutexServerUp.unlock()
        return True

    def cycleCheckServerUp(self):
        """
        :return: nothing
        """
        if self.host is None:
            return False

        if not self.mutexServerUp.tryLock():
            return False

        worker = Worker(self.checkServerUp)
        worker.signals.finished.connect(self.clearCycleCheckServerUp)
        worker.signals.result.connect(self.checkServerUpResult)
        worker.signals.error.connect(self.errorCycleCheckServerUp)
        self.threadPool.start(worker)
        return True

    def startTimers(self):
        """
        :return: nothing
        """
        self.timerServerUp.start(self.CYCLE_SERVER_UP)
        return True

    def stopTimers(self):
        """
        :return: nothing
        """
        self.timerServerUp.stop()
        self.threadPool.waitForDone()
        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import logging

# external packages

# local imports


class DirectWeather:
    """
    the class DirectWeather inherits all information and handling of the environment device

        >>> DirectWeather(host=None,
        >>>         name=''
        >>>         )
    """

    __all__ = ['DirectWeather',
               ]

    log = logging.getLogger(__name__)

    def __init__(self, app=None):

        self.app = app

        # minimum set for driver package built in
        self.framework = ''
        self.run = {
            'internal': self
        }
        self.deviceName = ''
        self.data = {}
        self.defaultConfig = {'framework': '',
                              'frameworks': {'internal': {'deviceName': 'Direct'}}}

    @staticmethod
    def startCommunication(loadConfig=False):
        """
        startCommunication enables the cyclic polling in framework driver

        :param loadConfig:
        :return: success
        """

        return True

    @staticmethod
    def stopCommunication():

        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries

# external packages
from PyQt5.QtCore import QObject, Qt, QRectF
from PyQt5.QtGui import QPixmap, QColor, QLinearGradient
from PyQt5.QtWidgets import QApplication, QSplashScreen
import numpy as np

# local import
import resource.resources
resource.resources.qInitResources()


class SplashScreen(QObject):
    """
    Splash screen show an icon with a progress bar and could send messages to the text
    set in the progress bar. Need the app and the icon as parameter

    Part from Maurizio D'Addona <mauritiusdadd@gmail.com> under license APL2.0
    Ported from PyQt4 to PyQt5

    Agreement for License (email from 04.07.2018):
    Hi Michel,
    sure, there is no problem for me. I'm glad you have found it useful.
    Best regards,
    Maurizio
    """

    __all__ = ['SplashScreen']

    def __init__(self, application=None):
        super().__init__()

        self._qapp = application
        self._pxm = QPixmap(':/icon/mw4.ico')

        flags = (Qt.WindowStaysOnTopHint | Qt.X11BypassWindowManagerHint)
        self.qss = QSplashScreen(self._pxm, flags)
        self.msg = ''
        self.maxv = 100.0
        self.minv = 0.0
        self.cval = 0.0
        self.qss.__drawContents__ = self.qss.drawContents
        self.qss.drawContents = self.drawContents
        self.qss.show()
        self.qss.raise_()
        QApplication.processEvents()

    def close(self):
        self.update()
        self.qss.close()

    def setValue(self, val):
        for i in np.arange(self.cval, val, self.maxv / 5.0):
            self.cval = i
            self.update()

    def showMessage(self, msg):
        self.msg = msg
        self.update()

    def update(self):
        self.qss.update()
        QApplication.processEvents()

    def drawContents(self, painter):
        view_port = painter.viewport()
        w = view_port.right()
        h = view_port.bottom()

        painter.setPen(QColor(55, 55, 55, 255))
        painter.setBrush(QColor(0, 0, 0, 255))
        painter.drawRect(10,
                         h - 64,
                         w - 20,
                         19)

        redlg = QLinearGradient(0, h - 63, 0, h)
        redlg.setColorAt(0.3, QColor(8, 36, 48))
        redlg.setColorAt(0, QColor(32, 144, 192))

        painter.setPen(Qt.NoPen)
        painter.setBrush(redlg)
        painter.drawRect(13,
                         h - 61,
                         int((w - 24) * self.cval / self.maxv),
                         14)

        painter.setPen(Qt.white)

        rect = QRectF(10,
                      h - 61,
                      w - 20,
                      15)

        painter.drawText(rect, Qt.AlignCenter, str(self.msg))

    def finish(self, qwid):
        self.qss.finish(qwid)



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import socket

# external packages
import PyQt5.QtCore
import PyQt5.QtWidgets
import wakeonlan

# local imports
from base.tpool import Worker
import mountcontrol.mount


__all__ = ['Mount',
           'MountSignals',
           ]


class MountSignals(PyQt5.QtCore.QObject):
    """
    The MountSignals class offers a list of signals to be used and instantiated by
    the Mount class to get signals for triggers for finished tasks to
    enable a gui to update their values transferred to the caller back.

    This has to be done in a separate class as the signals have to be subclassed from
    QObject and the Mount class itself is subclassed from object
    """

    __all__ = ['MountSignals']

    pointDone = PyQt5.QtCore.pyqtSignal(object)
    settingDone = PyQt5.QtCore.pyqtSignal(object)
    alignDone = PyQt5.QtCore.pyqtSignal(object)
    namesDone = PyQt5.QtCore.pyqtSignal(object)
    firmwareDone = PyQt5.QtCore.pyqtSignal(object)
    locationDone = PyQt5.QtCore.pyqtSignal(object)
    calcTLEdone = PyQt5.QtCore.pyqtSignal(object)
    statTLEdone = PyQt5.QtCore.pyqtSignal(object)
    getTLEdone = PyQt5.QtCore.pyqtSignal(object)
    mountUp = PyQt5.QtCore.pyqtSignal(object)
    slewFinished = PyQt5.QtCore.pyqtSignal()
    alert = PyQt5.QtCore.pyqtSignal()


class Mount(mountcontrol.mount.Mount):
    """
    The Mount class is the main interface for interacting with the mount computer.
    The user could:
        setup / change the interface to the mount
        start / stop cyclic tasks to poll data from mount
        send and get data from mount
        has signals for interfacing to external GUI's for data updates

        >>> settings = Mount(
        >>>                     host=host,
        >>>                     MAC=MAC,
        >>>                     threadPool=threadPool,
        >>>                     pathToData=pathToData,
        >>>                     expire=expire,
        >>>                     verbose=verbose,
        >>>                 )

    """

    CYCLE_POINTING = 500
    CYCLE_MOUNT_UP = 3000
    CYCLE_SETTING = 3000

    # set timeout
    SOCKET_TIMEOUT = 3.5

    def __init__(self,
                 host=None,
                 MAC=None,
                 threadPool=None,
                 pathToData=None,
                 verbose=None,
                 ):

        super().__init__(host=host,
                         MAC=MAC,
                         pathToData=pathToData,
                         verbose=verbose,
                         )

        if threadPool is None:
            self.threadPool = PyQt5.QtCore.QThreadPool()

        else:
            self.threadPool = threadPool

        self.mountUp = False
        self._settlingTime = 0
        self.statusAlert = False
        self.statusSlew = True
        self.signals = MountSignals()

        self.timerPointing = PyQt5.QtCore.QTimer()
        self.timerPointing.setSingleShot(False)
        self.timerPointing.timeout.connect(self.cyclePointing)
        self.timerSetting = PyQt5.QtCore.QTimer()
        self.timerSetting.setSingleShot(False)
        self.timerSetting.timeout.connect(self.cycleSetting)
        self.timerMountUp = PyQt5.QtCore.QTimer()
        self.timerMountUp.setSingleShot(False)
        self.timerMountUp.timeout.connect(self.cycleCheckMountUp)
        self.settlingWait = PyQt5.QtCore.QTimer()
        self.settlingWait.setSingleShot(True)
        self.settlingWait.timeout.connect(self.waitSettlingAndEmit)

    @property
    def settlingTime(self):
        return self._settlingTime / 1000

    @settlingTime.setter
    def settlingTime(self, value):
        self._settlingTime = value * 1000

    def waitSettlingAndEmit(self):
        """
        :return: true for test purpose
        """
        self.signals.slewFinished.emit()
        return True

    def startTimers(self):
        """
        :return: true for test purpose
        """
        self.timerSetting.start(self.CYCLE_SETTING)
        self.timerPointing.start(self.CYCLE_POINTING)
        self.timerMountUp.start(self.CYCLE_MOUNT_UP)
        return True

    def stopTimers(self):
        """
        :return: true for test purpose
        """
        self.timerPointing.stop()
        self.timerSetting.stop()
        self.timerMountUp.stop()
        self.threadPool.waitForDone()
        return True

    def resetData(self):
        """
        :return: true for test purpose
        """
        super().resetData()
        self.signals.pointDone.emit(self.obsSite)
        self.signals.settingDone.emit(self.setting)
        self.signals.alignDone.emit(self.model)
        self.signals.namesDone.emit(self.model)
        self.signals.firmwareDone.emit(self.firmware)
        self.signals.locationDone.emit(self.obsSite.location)
        return True

    def checkMountUp(self):
        """
        :return: true for test purpose
        """
        client = socket.socket()
        client.settimeout(self.SOCKET_TIMEOUT)
        try:
            client.connect(self.host)

        except Exception:
            self.mountUp = False

        else:
            self.mountUp = True
            # client.shutdown(socket.SHUT_RDWR)
            client.close()

        return self.mountUp

    def errorCycleCheckMountUp(self, e):
        """
        :return: true for test purpose
        """
        self.log.info(f'Cycle error: {e}')
        return True

    def clearCycleCheckMountUp(self):
        """
        :return: true for test purpose
        """
        self.signals.mountUp.emit(self.mountUp)
        return True

    def cycleCheckMountUp(self):
        """
        :return: success
        """
        if not self.host:
            self.signals.mountUp.emit(False)
            return False

        worker = Worker(self.checkMountUp)
        worker.signals.finished.connect(self.clearCycleCheckMountUp)
        worker.signals.error.connect(self.errorCycleCheckMountUp)
        self.threadPool.start(worker)
        return True

    def errorCyclePointing(self, e):
        """
        :return: true for test purpose
        """
        self.log.warning(f'Cycle error: {e}')
        return True

    def clearCyclePointing(self):
        """
        the cyclic or long lasting tasks for getting date from the mount should not run
        twice for the same data at the same time.

        :return: true for test purpose
        """
        if self.obsSite.status in [1, 98, 99]:
            if not self.statusAlert:
                self.signals.alert.emit()

            self.statusAlert = True

        else:
            self.statusAlert = False

        if self.obsSite.status not in [2, 6]:
            if not self.statusSlew:
                self.settlingWait.start(self._settlingTime)

            self.statusSlew = True

        else:
            self.statusSlew = False

        self.signals.pointDone.emit(self.obsSite)
        return True

    def cyclePointing(self):
        """
        :return: success
        """

        if not self.mountUp:
            self.signals.pointDone.emit(self.obsSite)
            return False

        worker = Worker(self.obsSite.pollPointing)
        worker.signals.finished.connect(self.clearCyclePointing)
        worker.signals.error.connect(self.errorCyclePointing)
        self.threadPool.start(worker)
        return True

    def errorCycleSetting(self, e):
        """
        :return: true for test purpose
        """
        self.log.warning(f'Cycle error: {e}')
        return True

    def clearCycleSetting(self):
        """
        :return: true for test purpose
        """
        self.signals.settingDone.emit(self.setting)
        return True

    def cycleSetting(self):
        """
        cycleSet prepares the worker thread and the signals for getting the settings
        data.

        :return: success
        """
        if not self.mountUp:
            self.signals.settingDone.emit(self.setting)
            return False

        worker = Worker(self.setting.pollSetting)
        worker.signals.finished.connect(self.clearCycleSetting)
        worker.signals.error.connect(self.errorCycleSetting)
        self.threadPool.start(worker)
        return True

    def errorGetAlign(self, e):
        """
        :return: true for test purpose
        """
        self.log.warning(f'Cycle error: {e}')
        return True

    def clearGetAlign(self):
        """
        :return: true for test purpose
        """
        self.signals.alignDone.emit(self.model)
        return True

    def getAlign(self):
        """
        getAlign prepares the worker thread and the signals for getting the alignment model
        data.

        :return: success
        """
        if not self.mountUp:
            self.signals.alignDone.emit(self.model)
            return False

        worker = Worker(self.model.pollStars)
        worker.signals.finished.connect(self.clearGetAlign)
        worker.signals.error.connect(self.errorGetAlign)
        self.threadPool.start(worker)
        return True

    def errorGetNames(self, e):
        """
        :return: true for test purpose
        """
        self.log.warning(f'Cycle error: {e}')
        return True

    def clearGetNames(self):
        """
        :return: true for test purpose
        """
        self.signals.namesDone.emit(self.model)
        return True

    def getNames(self):
        """
        getNames prepares the worker thread and the signals for getting the alignment model
        names.

        :return: success
        """
        if not self.mountUp:
            self.signals.namesDone.emit(self.model)
            return False

        worker = Worker(self.model.pollNames)
        worker.signals.finished.connect(self.clearGetNames)
        worker.signals.error.connect(self.errorGetNames)
        self.threadPool.start(worker)
        return True

    def errorGetFW(self, e):
        """
        :return: true for test purpose
        """
        self.log.warning(f'Cycle error: {e}')
        return True

    def clearGetFW(self):
        """
        :return: true for test purpose
        """
        self.geometry.initializeGeometry(self.firmware.product)
        self.signals.firmwareDone.emit(self.firmware)
        return True

    def getFW(self):
        """
        getFW prepares the worker thread and the signals for getting the build data of
        the mount computer.

        :return: success
        """
        if not self.mountUp:
            self.signals.firmwareDone.emit(self.firmware)
            return False

        worker = Worker(self.firmware.poll)
        worker.signals.finished.connect(self.clearGetFW)
        worker.signals.error.connect(self.errorGetFW)
        self.threadPool.start(worker)
        return True

    def errorGetLocation(self, e):
        """
        :return: true for test purpose
        """
        self.log.warning(f'Cycle error: {e}')
        return True

    def clearGetLocation(self):
        """
        :return: true for test purpose
        """
        self.signals.locationDone.emit(self.obsSite)
        return True

    def getLocation(self):
        """
        getLocation prepares the worker thread and the signals for getting the mount
        location data.

        :return: success
        """
        if not self.mountUp:
            self.signals.locationDone.emit(self.obsSite)
            return False

        worker = Worker(self.obsSite.getLocation)
        worker.signals.finished.connect(self.clearGetLocation)
        worker.signals.error.connect(self.errorGetLocation)
        self.threadPool.start(worker)
        return True

    def errorCalcTLE(self, e):
        """
        :return: true for test purpose
        """
        self.log.warning(f'Cycle error: {e}')
        return True

    def clearCalcTLE(self):
        """
        :return: true for test purpose
        """
        self.signals.calcTLEdone.emit(self.satellite.tleParams)
        return True

    def calcTLE(self):
        """
        getCalcTLE prepares the worker thread and the signals for getting the mount
        location data.

        :return: success
        """
        if not self.mountUp:
            self.signals.calcTLEdone.emit(self.satellite.tleParams)
            return False

        worker = Worker(self.satellite.calcTLE, self.obsSite.timeJD.tt)
        worker.signals.finished.connect(self.clearCalcTLE)
        worker.signals.error.connect(self.errorCalcTLE)
        self.threadPool.start(worker)
        return True

    def errorStatTLE(self, e):
        """
        :return: true for test purpose
        """
        self.log.warning(f'Cycle error: {e}')
        return True

    def clearStatTLE(self):
        """
        :return: true for test purpose
        """
        self.signals.statTLEdone.emit(self.satellite.tleParams)
        return True

    def statTLE(self):
        """
        statTLE prepares the worker thread and the signals for getting the mount
        location data.

        :return: success
        """
        if not self.mountUp:
            self.signals.statTLEdone.emit(self.satellite.tleParams)
            return False

        worker = Worker(self.satellite.statTLE)
        worker.signals.finished.connect(self.clearStatTLE)
        worker.signals.error.connect(self.errorStatTLE)
        self.threadPool.start(worker)
        return True

    def errorGetTLE(self, e):
        """
        :return: true for test purpose
        """
        self.log.warning(f'Cycle error: {e}')
        return True

    def clearGetTLE(self):
        """
        :return: true for test purpose
        """
        self.signals.getTLEdone.emit(self.satellite.tleParams)
        return True

    def getTLE(self):
        """
        getTLE prepares the worker thread and the signals for getting the mount
        location data.

        :return: success
        """
        if not self.mountUp:
            self.signals.getTLEdone.emit(self.satellite.tleParams)
            return False

        worker = Worker(self.satellite.getTLE)
        worker.signals.finished.connect(self.clearGetTLE)
        worker.signals.error.connect(self.errorGetTLE)
        self.threadPool.start(worker)
        return True

    def bootMount(self):
        """
        :return:    True if success
        """

        if self.MAC is not None:
            wakeonlan.send_magic_packet(self.MAC)
            return True

        else:
            return False

    def shutdown(self):
        """
        :return:
        """

        suc = self.obsSite.shutdown()
        if suc:
            self.mountUp = False

        return suc
